@@ -1,6 +1,5 @@
 # --
-# Kernel/Modules/AdminGenericInterfaceOperationDefault.pm - provides a log view for admins
-# Copyright (C) 2001-2014 OTRS AG, http://otrs.com/
+# Copyright (C) 2001-2017 OTRS AG, http://otrs.com/
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
 # the enclosed file COPYING for license information (AGPL). If you
@@ -12,9 +11,10 @@ package Kernel::Modules::AdminGenericInterfaceOperationDefault;
 use strict;
 use warnings;
 
-use Kernel::System::GenericInterface::Webservice;
-
 use Kernel::System::VariableCheck qw(:all);
+use Kernel::Language qw(Translatable);
+
+our $ObjectManagerDisabled = 1;
 
 sub new {
     my ( $Type, %Param ) = @_;
@@ -22,32 +22,41 @@ sub new {
     my $Self = {%Param};
     bless( $Self, $Type );
 
-    for (qw(ParamObject LayoutObject LogObject ConfigObject)) {
-        if ( !$Self->{$_} ) {
-            $Self->{LayoutObject}->FatalError( Message => "Got no $_!" );
-        }
-    }
-
-    $Self->{WebserviceObject} = Kernel::System::GenericInterface::Webservice->new( %{$Self} );
-
     return $Self;
 }
 
 sub Run {
     my ( $Self, %Param ) = @_;
 
-    my $WebserviceID = int( $Self->{ParamObject}->GetParam( Param => 'WebserviceID' ) || 0 );
-    if ( !$WebserviceID ) {
-        return $Self->{LayoutObject}->ErrorScreen(
-            Message => "Need WebserviceID!",
+    my $LayoutObject = $Kernel::OM->Get('Kernel::Output::HTML::Layout');
+
+    my $WebserviceID = $Kernel::OM->Get('Kernel::System::Web::Request')->GetParam( Param => 'WebserviceID' );
+    if ( !IsStringWithData($WebserviceID) ) {
+        return $LayoutObject->ErrorScreen(
+            Message => Translatable('Need WebserviceID!'),
         );
     }
 
-    my $WebserviceData = $Self->{WebserviceObject}->WebserviceGet( ID => $WebserviceID );
+    if ( !$WebserviceID ) {
+        return $LayoutObject->ErrorScreen(
+            Message => Translatable('Need WebserviceID!'),
+        );
+    }
+
+    # send data to JS
+    $LayoutObject->AddJSData(
+        Key   => 'WebserviceID',
+        Value => $WebserviceID
+    );
+
+    my $WebserviceData = $Kernel::OM->Get('Kernel::System::GenericInterface::Webservice')->WebserviceGet(
+        ID => $WebserviceID,
+    );
 
     if ( !IsHashRefWithData($WebserviceData) ) {
-        return $Self->{LayoutObject}->ErrorScreen(
-            Message => "Could not get data for WebserviceID $WebserviceID",
+        return $LayoutObject->ErrorScreen(
+            Message =>
+                $LayoutObject->{LanguageObject}->Translate( 'Could not get data for WebserviceID %s', $WebserviceID ),
         );
     }
 
@@ -60,8 +69,8 @@ sub Run {
     }
     elsif ( $Self->{Subaction} eq 'AddAction' ) {
 
-        # challenge token check for write action
-        $Self->{LayoutObject}->ChallengeTokenCheck();
+        # Challenge token check for write action.
+        $LayoutObject->ChallengeTokenCheck();
 
         return $Self->_AddAction(
             %Param,
@@ -78,8 +87,8 @@ sub Run {
     }
     elsif ( $Self->{Subaction} eq 'ChangeAction' ) {
 
-        # challenge token check for write action
-        $Self->{LayoutObject}->ChallengeTokenCheck();
+        # Challenge token check for write action.
+        $LayoutObject->ChallengeTokenCheck();
 
         return $Self->_ChangeAction(
             %Param,
@@ -89,8 +98,8 @@ sub Run {
     }
     elsif ( $Self->{Subaction} eq 'DeleteAction' ) {
 
-        # challenge token check for write action
-        $Self->{LayoutObject}->ChallengeTokenCheck();
+        # Challenge token check for write action.
+        $LayoutObject->ChallengeTokenCheck();
 
         return $Self->_DeleteAction(
             %Param,
@@ -107,22 +116,24 @@ sub _Add {
     my $WebserviceID   = $Param{WebserviceID};
     my $WebserviceData = $Param{WebserviceData};
 
-    my $OperationType = $Self->{ParamObject}->GetParam( Param => 'OperationType' );
+    my $OperationType = $Kernel::OM->Get('Kernel::System::Web::Request')->GetParam( Param => 'OperationType' );
+
+    my $LayoutObject = $Kernel::OM->Get('Kernel::Output::HTML::Layout');
 
     if ( !$OperationType ) {
-        return $Self->{LayoutObject}->ErrorScreen(
-            Message => "Need OperationType",
+        return $LayoutObject->ErrorScreen(
+            Message => Translatable('Need OperationType'),
         );
     }
     if ( !$Self->_OperationTypeCheck( OperationType => $OperationType ) ) {
-        return $Self->{LayoutObject}->ErrorScreen(
-            Message => "Operation $OperationType is not registered",
+        return $LayoutObject->ErrorScreen(
+            Message => $LayoutObject->{LanguageObject}->Translate( 'Operation %s is not registered', $OperationType ),
         );
     }
 
     return $Self->_ShowScreen(
         %Param,
-        Mode           => 'Add',
+        Action         => 'Add',
         WebserviceID   => $WebserviceID,
         WebserviceData => $WebserviceData,
         WebserviceName => $WebserviceData->{Name},
@@ -139,43 +150,48 @@ sub _AddAction {
     my %Errors;
     my %GetParam;
 
+    my $ParamObject = $Kernel::OM->Get('Kernel::System::Web::Request');
+
     for my $Needed (qw(Operation OperationType)) {
-        $GetParam{$Needed} = $Self->{ParamObject}->GetParam( Param => $Needed );
+        $GetParam{$Needed} = $ParamObject->GetParam( Param => $Needed );
         if ( !$GetParam{$Needed} ) {
             $Errors{ $Needed . 'ServerError' } = 'ServerError';
         }
     }
 
-    # name already exists, bail out
+    # Name already exists, bail out.
     if ( exists $WebserviceData->{Config}->{Provider}->{Operation}->{ $GetParam{Operation} } ) {
         $Errors{OperationServerError} = 'ServerError';
     }
 
-    # uncorrectable errors
+    my $LayoutObject = $Kernel::OM->Get('Kernel::Output::HTML::Layout');
+
+    # Uncorrectable errors.
     if ( !$GetParam{OperationType} ) {
-        return $Self->{LayoutObject}->ErrorScreen(
-            Message => "Need OperationType",
+        return $LayoutObject->ErrorScreen(
+            Message => Translatable('Need OperationType'),
         );
     }
     if ( !$Self->_OperationTypeCheck( OperationType => $GetParam{OperationType} ) ) {
-        return $Self->{LayoutObject}->ErrorScreen(
-            Message => "OperationType $GetParam{OperationType} is not registered",
+        return $LayoutObject->ErrorScreen(
+            Message => $LayoutObject->{LanguageObject}
+                ->Translate( 'OperationType %s is not registered', $GetParam{OperationType} ),
         );
     }
 
-    # validation errors
+    # Validation errors.
     if (%Errors) {
 
         # get the description from the web request to send it back again to the screen
         my $OperationConfig;
-        $OperationConfig->{Description} = $Self->{ParamObject}->GetParam( Param => 'Description' )
-            || '';
+        $OperationConfig->{Description}       = $ParamObject->GetParam( Param => 'Description' )       || '';
+        $OperationConfig->{IncludeTicketData} = $ParamObject->GetParam( Param => 'IncludeTicketData' ) || '';
 
         return $Self->_ShowScreen(
             %Param,
             %GetParam,
             %Errors,
-            Mode            => 'Add',
+            Action          => 'Add',
             WebserviceID    => $WebserviceID,
             WebserviceData  => $WebserviceData,
             WebserviceName  => $WebserviceData->{Name},
@@ -185,11 +201,12 @@ sub _AddAction {
     }
 
     my $Config = {
-        Type        => $GetParam{OperationType},
-        Description => $Self->{ParamObject}->GetParam( Param => 'Description' ) || '',
+        Type              => $GetParam{OperationType},
+        Description       => $ParamObject->GetParam( Param => 'Description' ) || '',
+        IncludeTicketData => $ParamObject->GetParam( Param => 'IncludeTicketData' ) || '',
     };
 
-    my $MappingInbound = $Self->{ParamObject}->GetParam( Param => 'MappingInbound' );
+    my $MappingInbound = $ParamObject->GetParam( Param => 'MappingInbound' );
     $MappingInbound = $Self->_MappingTypeCheck( MappingType => $MappingInbound ) ? $MappingInbound : '';
 
     if ($MappingInbound) {
@@ -198,7 +215,7 @@ sub _AddAction {
         };
     }
 
-    my $MappingOutbound = $Self->{ParamObject}->GetParam( Param => 'MappingOutbound' );
+    my $MappingOutbound = $ParamObject->GetParam( Param => 'MappingOutbound' );
     $MappingOutbound = $Self->_MappingTypeCheck( MappingType => $MappingOutbound ) ? $MappingOutbound : '';
 
     if ($MappingOutbound) {
@@ -207,21 +224,20 @@ sub _AddAction {
         };
     }
 
-    # check if Operation already exists
+    # Check if Operation already exists.
     if ( !$WebserviceData->{Config}->{Provider}->{Operation}->{ $GetParam{Operation} } ) {
         $WebserviceData->{Config}->{Provider}->{Operation}->{ $GetParam{Operation} } = $Config;
 
-        $Self->{WebserviceObject}->WebserviceUpdate(
+        $Kernel::OM->Get('Kernel::System::GenericInterface::Webservice')->WebserviceUpdate(
             %{$WebserviceData},
             UserID => $Self->{UserID},
         );
     }
 
     my $RedirectURL = "Action=AdminGenericInterfaceOperationDefault;Subaction=Change;WebserviceID=$WebserviceID;";
-    $RedirectURL
-        .= 'Operation=' . $Self->{LayoutObject}->LinkEncode( $GetParam{Operation} ) . ';';
+    $RedirectURL .= 'Operation=' . $LayoutObject->LinkEncode( $GetParam{Operation} ) . ';';
 
-    return $Self->{LayoutObject}->Redirect(
+    return $LayoutObject->Redirect(
         OP => $RedirectURL,
     );
 }
@@ -232,11 +248,13 @@ sub _Change {
     my $WebserviceID   = $Param{WebserviceID};
     my $WebserviceData = $Param{WebserviceData};
 
-    my $Operation = $Self->{ParamObject}->GetParam( Param => 'Operation' );
+    my $Operation = $Kernel::OM->Get('Kernel::System::Web::Request')->GetParam( Param => 'Operation' );
+
+    my $LayoutObject = $Kernel::OM->Get('Kernel::Output::HTML::Layout');
 
     if ( !$Operation ) {
-        return $Self->{LayoutObject}->ErrorScreen(
-            Message => "Need Operation",
+        return $LayoutObject->ErrorScreen(
+            Message => Translatable('Need Operation'),
         );
     }
 
@@ -247,8 +265,9 @@ sub _Change {
         || ref $WebserviceData->{Config}->{Provider}->{Operation}->{$Operation} ne 'HASH'
         )
     {
-        return $Self->{LayoutObject}->ErrorScreen(
-            Message => "Could not determine config for operation $Operation",
+        return $LayoutObject->ErrorScreen(
+            Message =>
+                $LayoutObject->{LanguageObject}->Translate( 'Could not determine config for operation %s', $Operation ),
         );
     }
 
@@ -256,7 +275,7 @@ sub _Change {
 
     return $Self->_ShowScreen(
         %Param,
-        Mode            => 'Change',
+        Action          => 'Change',
         WebserviceID    => $WebserviceID,
         WebserviceData  => $WebserviceData,
         WebserviceName  => $WebserviceData->{Name},
@@ -276,13 +295,17 @@ sub _ChangeAction {
     my $WebserviceID   = $Param{WebserviceID};
     my $WebserviceData = $Param{WebserviceData};
 
+    # get needed objects
+    my $ParamObject  = $Kernel::OM->Get('Kernel::System::Web::Request');
+    my $LayoutObject = $Kernel::OM->Get('Kernel::Output::HTML::Layout');
+
     for my $Needed (qw(Operation OldOperation)) {
 
-        $GetParam{$Needed} = $Self->{ParamObject}->GetParam( Param => $Needed );
+        $GetParam{$Needed} = $ParamObject->GetParam( Param => $Needed );
 
         if ( !$GetParam{$Needed} ) {
-            return $Self->{LayoutObject}->ErrorScreen(
-                Message => "Need $Needed",
+            return $LayoutObject->ErrorScreen(
+                Message => $LayoutObject->{LanguageObject}->Translate( 'Need %s', $Needed ),
             );
         }
     }
@@ -296,22 +319,23 @@ sub _ChangeAction {
         'HASH'
         )
     {
-        return $Self->{LayoutObject}->ErrorScreen(
-            Message => "Could not determine config for operation $GetParam{OldOperation}",
+        return $LayoutObject->ErrorScreen(
+            Message => $LayoutObject->{LanguageObject}
+                ->Translate( 'Could not determine config for operation %s', $GetParam{OldOperation} ),
         );
     }
 
     my $OperationConfig = $WebserviceData->{Config}->{Provider}->{Operation}->{ $GetParam{OldOperation} };
 
-    # Operation was renamed, avoid conflicts
+    # Operation was renamed, avoid conflicts.
     if ( $GetParam{OldOperation} ne $GetParam{Operation} ) {
 
-        # New name already exists, bail out
+        # New name already exists, bail out.
         if ( exists $WebserviceData->{Config}->{Provider}->{Operation}->{ $GetParam{Operation} } ) {
             $Errors{OperationServerError} = 'ServerError';
         }
 
-        # Ok, remove old Operation. New one will be added below.
+        # OK, remove old Operation. New one will be added below.
         if ( !%Errors ) {
             delete $WebserviceData->{Config}->{Provider}->{Operation}->{ $GetParam{OldOperation} };
         }
@@ -323,7 +347,7 @@ sub _ChangeAction {
             %Param,
             %GetParam,
             %Errors,
-            Mode            => 'Change',
+            Action          => 'Change',
             WebserviceID    => $WebserviceID,
             WebserviceData  => $WebserviceData,
             WebserviceName  => $WebserviceData->{Name},
@@ -335,7 +359,7 @@ sub _ChangeAction {
     }
 
     # Now handle mappings. If mapping types were not changed, keep the mapping configuration.
-    my $MappingInbound = $Self->{ParamObject}->GetParam( Param => 'MappingInbound' );
+    my $MappingInbound = $ParamObject->GetParam( Param => 'MappingInbound' );
     $MappingInbound = $Self->_MappingTypeCheck( MappingType => $MappingInbound ) ? $MappingInbound : '';
 
     # No inbound mapping set, make sure it is not present in the configuration.
@@ -351,7 +375,7 @@ sub _ChangeAction {
         };
     }
 
-    my $MappingOutbound = $Self->{ParamObject}->GetParam( Param => 'MappingOutbound' );
+    my $MappingOutbound = $ParamObject->GetParam( Param => 'MappingOutbound' );
     $MappingOutbound = $Self->_MappingTypeCheck( MappingType => $MappingOutbound ) ? $MappingOutbound : '';
 
     # No outbound mapping set, make sure it is not present in the configuration.
@@ -367,29 +391,62 @@ sub _ChangeAction {
         };
     }
 
-    $OperationConfig->{Description} = $Self->{ParamObject}->GetParam( Param => 'Description' )
-        || '';
+    $OperationConfig->{Description}       = $ParamObject->GetParam( Param => 'Description' )       || '';
+    $OperationConfig->{IncludeTicketData} = $ParamObject->GetParam( Param => 'IncludeTicketData' ) || '';
 
     # Update operation config.
     $WebserviceData->{Config}->{Provider}->{Operation}->{ $GetParam{Operation} } = $OperationConfig;
 
+    # Take care of error handlers with operation filters.
+    ERRORHANDLING:
+    for my $ErrorHandling ( sort keys %{ $Param{WebserviceData}->{Config}->{Provider}->{ErrorHandling} || {} } ) {
+
+        if ( !IsHashRefWithData( $Param{WebserviceData}->{Config}->{Provider}->{ErrorHandling}->{$ErrorHandling} ) ) {
+            next ERRORHANDLING;
+        }
+        my $OperationFilter
+            = $Param{WebserviceData}->{Config}->{Provider}->{ErrorHandling}->{$ErrorHandling}->{OperationFilter};
+        next ERRORHANDLING if !IsArrayRefWithData($OperationFilter);
+        next ERRORHANDLING if !grep { $_ eq $GetParam{OldOperation} } @{$OperationFilter};
+
+        # Rename operation in error handling operation filter as well to keep consistency.
+        my @NewOperationFilter = map { $_ eq $GetParam{OldOperation} ? $GetParam{Operation} : $_ } @{$OperationFilter};
+        $Param{WebserviceData}->{Config}->{Provider}->{ErrorHandling}->{$ErrorHandling}->{OperationFilter}
+            = \@NewOperationFilter;
+    }
+
     # Write new config to database.
-    $Self->{WebserviceObject}->WebserviceUpdate(
+    $Kernel::OM->Get('Kernel::System::GenericInterface::Webservice')->WebserviceUpdate(
         %{$WebserviceData},
         UserID => $Self->{UserID},
     );
 
-    # Save button: stay in edit mode.
-    my $RedirectURL
-        = "Action=AdminGenericInterfaceOperationDefault;Subaction=Change;WebserviceID=$WebserviceID;Operation=$GetParam{Operation}";
-
-    # Save and finish button: go to Webservice.
-    if ( $Self->{ParamObject}->GetParam( Param => 'ReturnToWebservice' ) ) {
-        $RedirectURL = "Action=AdminGenericInterfaceWebservice;Subaction=Change;WebserviceID=$WebserviceID;";
+    # If the user would like to continue editing the invoker config, just redirect to the edit screen.
+    my $RedirectURL;
+    if (
+        defined $ParamObject->GetParam( Param => 'ContinueAfterSave' )
+        && ( $ParamObject->GetParam( Param => 'ContinueAfterSave' ) eq '1' )
+        )
+    {
+        $RedirectURL =
+            'Action='
+            . $Self->{Action}
+            . ';Subaction=Change;WebserviceID='
+            . $WebserviceID
+            . ';Operation='
+            . $LayoutObject->LinkEncode( $GetParam{Operation} )
+            . ';';
+    }
+    else {
 
+        # Otherwise return to overview.
+        $RedirectURL =
+            'Action=AdminGenericInterfaceWebservice;Subaction=Change;WebserviceID='
+            . $WebserviceID
+            . ';';
     }
 
-    return $Self->{LayoutObject}->Redirect(
+    return $LayoutObject->Redirect(
         OP => $RedirectURL,
     );
 }
@@ -399,10 +456,10 @@ sub _DeleteAction {
 
     my $WebserviceData = $Param{WebserviceData};
 
-    my $Operation = $Self->{ParamObject}->GetParam( Param => 'Operation' );
+    my $Operation = $Kernel::OM->Get('Kernel::System::Web::Request')->GetParam( Param => 'Operation' );
 
     if ( !$Operation ) {
-        $Self->{LogObject}->Log(
+        $Kernel::OM->Get('Kernel::System::Log')->Log(
             Priority => 'error',
             Message  => 'Got no Operation',
         );
@@ -414,22 +471,24 @@ sub _DeleteAction {
     if ( $WebserviceData->{Config}->{Provider}->{Operation}->{$Operation} ) {
         delete $WebserviceData->{Config}->{Provider}->{Operation}->{$Operation};
 
-        $Success = $Self->{WebserviceObject}->WebserviceUpdate(
+        $Success = $Kernel::OM->Get('Kernel::System::GenericInterface::Webservice')->WebserviceUpdate(
             %{$WebserviceData},
             UserID => $Self->{UserID},
         );
     }
 
-    # build JSON output
-    my $JSON = $Self->{LayoutObject}->JSONEncode(
+    my $LayoutObject = $Kernel::OM->Get('Kernel::Output::HTML::Layout');
+
+    # Build JSON output.
+    my $JSON = $LayoutObject->JSONEncode(
         Data => {
             Success => $Success,
         },
     );
 
-    # send JSON response
-    return $Self->{LayoutObject}->Attachment(
-        ContentType => 'application/json; charset=' . $Self->{LayoutObject}->{Charset},
+    # Send JSON response.
+    return $LayoutObject->Attachment(
+        ContentType => 'application/json; charset=' . $LayoutObject->{Charset},
         Content     => $JSON,
         Type        => 'inline',
         NoCache     => 1,
@@ -439,101 +498,111 @@ sub _DeleteAction {
 sub _ShowScreen {
     my ( $Self, %Param ) = @_;
 
-    my $Output = $Self->{LayoutObject}->Header();
-    $Output .= $Self->{LayoutObject}->NavigationBar();
+    # Get layout object.
+    my $LayoutObject = $Kernel::OM->Get('Kernel::Output::HTML::Layout');
 
-    $Self->{LayoutObject}->Block(
-        Name => 'Title' . $Param{Mode},
-        Data => \%Param
-    );
-    $Self->{LayoutObject}->Block(
-        Name => 'Navigation' . $Param{Mode},
-        Data => \%Param
-    );
+    my $Output = $LayoutObject->Header();
+    $Output .= $LayoutObject->NavigationBar();
+
+    # Send data to JS.
+    if ( $Param{Operation} ) {
+        $LayoutObject->AddJSData(
+            Key   => 'Operation',
+            Value => $Param{Operation},
+        );
+    }
 
     my %TemplateData;
 
-    if ( $Param{Mode} eq 'Add' ) {
+    if ( $Param{Action} eq 'Add' ) {
         $TemplateData{OperationType} = $Param{OperationType};
 
     }
-    elsif ( $Param{Mode} eq 'Change' ) {
-        $Self->{LayoutObject}->Block(
+    elsif ( $Param{Action} eq 'Change' ) {
+        $LayoutObject->Block(
             Name => 'ActionListDelete',
             Data => \%Param
         );
 
-        $Self->{LayoutObject}->Block(
-            Name => 'SaveAndFinishButton',
-            Data => \%Param
-        );
-
         $TemplateData{OperationType} = $Param{OperationConfig}->{Type};
     }
 
     $TemplateData{Description} = $Param{OperationConfig}->{Description};
 
-    my $Mappings = $Self->{ConfigObject}->Get('GenericInterface::Mapping::Module') || {};
+    my $Mappings = $Kernel::OM->Get('Kernel::Config')->Get('GenericInterface::Mapping::Module') || {};
 
-    # Inbound mapping
+    # Inbound mapping.
     my @MappingList = sort keys %{$Mappings};
 
     my $MappingInbound = $Self->_MappingTypeCheck( MappingType => $Param{MappingInbound} )
         ? $Param{MappingInbound}
         : '';
 
-    $TemplateData{MappingInboundStrg} = $Self->{LayoutObject}->BuildSelection(
+    $TemplateData{MappingInboundStrg} = $LayoutObject->BuildSelection(
         Data          => \@MappingList,
         Name          => 'MappingInbound',
         SelectedValue => $MappingInbound,
         Sort          => 'AlphanumericValue',
         PossibleNone  => 1,
-        Class         => 'RegisterChange',
+        Class         => 'Modernize RegisterChange',
     );
 
     if ($MappingInbound) {
         $TemplateData{MappingInboundConfigDialog} = $Mappings->{$MappingInbound}->{ConfigDialog};
     }
 
-    if ( $TemplateData{MappingInboundConfigDialog} && $Param{Mode} eq 'Change' ) {
-        $Self->{LayoutObject}->Block(
+    if ( $TemplateData{MappingInboundConfigDialog} && $Param{Action} eq 'Change' ) {
+        $LayoutObject->Block(
             Name => 'MappingInboundConfigureButton',
             Data => {
                 %Param,
                 %TemplateData,
-                }
+            },
         );
     }
 
-    # Outbound mapping
+    # Outbound mapping.
     my $MappingOutbound = $Self->_MappingTypeCheck( MappingType => $Param{MappingOutbound} )
         ? $Param{MappingOutbound}
         : '';
 
-    $TemplateData{MappingOutboundStrg} = $Self->{LayoutObject}->BuildSelection(
+    $TemplateData{MappingOutboundStrg} = $LayoutObject->BuildSelection(
         Data          => \@MappingList,
         Name          => 'MappingOutbound',
         SelectedValue => $MappingOutbound,
         Sort          => 'AlphanumericValue',
         PossibleNone  => 1,
-        Class         => 'RegisterChange',
+        Class         => 'Modernize RegisterChange',
     );
 
     if ($MappingOutbound) {
         $TemplateData{MappingOutboundConfigDialog} = $Mappings->{$MappingOutbound}->{ConfigDialog};
     }
 
-    if ( $TemplateData{MappingOutboundConfigDialog} && $Param{Mode} eq 'Change' ) {
-        $Self->{LayoutObject}->Block(
+    if ( $TemplateData{MappingOutboundConfigDialog} && $Param{Action} eq 'Change' ) {
+        $LayoutObject->Block(
             Name => 'MappingOutboundConfigureButton',
             Data => {
                 %Param,
                 %TemplateData,
-                }
+            },
         );
     }
 
-    $Output .= $Self->{LayoutObject}->Output(
+    $TemplateData{IncludeTicketDataStrg} = $LayoutObject->BuildSelection(
+        Data => {
+            0 => 'No',
+            1 => 'Yes',
+        },
+        Name       => 'IncludeTicketData',
+        SelectedID => $Param{OperationConfig}->{IncludeTicketData},
+        Sort       => 'NumericKey',
+        Class      => 'Modernize W50pc',
+        Disabled   => $TemplateData{OperationType} eq 'Ticket::TicketCreate'
+            || $TemplateData{OperationType} eq 'Ticket::TicketUpdate' ? 0 : 1,
+    );
+
+    $Output .= $LayoutObject->Output(
         TemplateFile => 'AdminGenericInterfaceOperationDefault',
         Data         => {
             %Param,
@@ -542,39 +611,39 @@ sub _ShowScreen {
         },
     );
 
-    $Output .= $Self->{LayoutObject}->Footer();
+    $Output .= $LayoutObject->Footer();
     return $Output;
 }
 
-=item _OperationTypeCheck()
-
-checks if a given OperationType is registered in the system.
-
-=cut
+# =head2 _OperationTypeCheck()
+#
+# checks if a given OperationType is registered in the system.
+#
+# =cut
 
 sub _OperationTypeCheck {
     my ( $Self, %Param ) = @_;
 
     return 0 if !$Param{OperationType};
 
-    my $Operations = $Self->{ConfigObject}->Get('GenericInterface::Operation::Module');
+    my $Operations = $Kernel::OM->Get('Kernel::Config')->Get('GenericInterface::Operation::Module');
     return 0 if ref $Operations ne 'HASH';
 
     return ref $Operations->{ $Param{OperationType} } eq 'HASH' ? 1 : 0;
 }
 
-=item _MappingTypeCheck()
-
-checks if a given MappingType is registered in the system.
-
-=cut
+# =head2 _MappingTypeCheck()
+#
+# checks if a given MappingType is registered in the system.
+#
+# =cut
 
 sub _MappingTypeCheck {
     my ( $Self, %Param ) = @_;
 
     return 0 if !$Param{MappingType};
 
-    my $Mappings = $Self->{ConfigObject}->Get('GenericInterface::Mapping::Module');
+    my $Mappings = $Kernel::OM->Get('Kernel::Config')->Get('GenericInterface::Mapping::Module');
     return 0 if ref $Mappings ne 'HASH';
 
     return ref $Mappings->{ $Param{MappingType} } eq 'HASH' ? 1 : 0;
