@@ -1,9 +1,9 @@
 # --
-# Copyright (C) 2001-2018 OTRS AG, https://otrs.com/
+# Copyright (C) 2001-2017 OTRS AG, http://otrs.com/
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
-# the enclosed file COPYING for license information (GPL). If you
-# did not receive this file, see https://www.gnu.org/licenses/gpl-3.0.txt.
+# the enclosed file COPYING for license information (AGPL). If you
+# did not receive this file, see http://www.gnu.org/licenses/agpl.txt.
 # --
 
 package Kernel::Modules::AdminOTRSBusiness;
@@ -43,7 +43,7 @@ sub Run {
             Data         => \%Param
         );
         $Output .= $LayoutObject->Footer();
-        return $Output;
+        return $Output
     }
 
     # get needed objects
@@ -241,6 +241,7 @@ sub NotInstalledScreen {
         if ( $GetParam{Type} ) {
             $LayoutObject->Block(
                 Name => 'Actions',
+                Data => \%GetParam,
             );
         }
 
