@@ -1,9 +1,9 @@
 # --
-# Copyright (C) 2001-2020 OTRS AG, https://otrs.com/
+# Copyright (C) 2001-2017 OTRS AG, http://otrs.com/
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
-# the enclosed file COPYING for license information (GPL). If you
-# did not receive this file, see https://www.gnu.org/licenses/gpl-3.0.txt.
+# the enclosed file COPYING for license information (AGPL). If you
+# did not receive this file, see http://www.gnu.org/licenses/agpl.txt.
 # --
 
 package Kernel::Output::HTML::Dashboard::Calendar;
@@ -143,14 +143,21 @@ sub Run {
                     next TICKETID;
                 }
 
-                # get time object
-                my $TimeObject = $Kernel::OM->Get('Kernel::System::Time');
+                # get current system datetime object
+                my $CurSystemDateTimeObject = $Kernel::OM->Create('Kernel::System::DateTime');
 
-                my $DestDate = $TimeObject->SystemTime() + $Ticket{UntilTime};
-                $TimeTill  = $Ticket{UntilTime};
-                $TimeStamp = $TimeObject->SystemTime2TimeStamp(
-                    SystemTime => $DestDate,
+                my $DestDate        = $CurSystemDateTimeObject->ToEpoch() + $Ticket{UntilTime};
+                my $TimeStampObject = $Kernel::OM->Create(
+                    'Kernel::System::DateTime',
+                    ObjectParams => {
+                        Epoch => $DestDate,
+                    },
                 );
+                if ($TimeStampObject) {
+                    $TimeStamp = $TimeStampObject->ToString();
+                }
+
+                $TimeTill = $Ticket{UntilTime};
             }
 
             # remember attributes for content table
