@@ -1,9 +1,9 @@
 # --
-# Copyright (C) 2001-2020 OTRS AG, https://otrs.com/
+# Copyright (C) 2001-2017 OTRS AG, http://otrs.com/
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
-# the enclosed file COPYING for license information (GPL). If you
-# did not receive this file, see https://www.gnu.org/licenses/gpl-3.0.txt.
+# the enclosed file COPYING for license information (AGPL). If you
+# did not receive this file, see http://www.gnu.org/licenses/agpl.txt.
 # --
 
 package Kernel::Output::HTML::HeaderMeta::Refresh;
@@ -28,12 +28,11 @@ sub new {
 sub Run {
     my ( $Self, %Param ) = @_;
 
-    if ( $Param{Refresh} ) {
-        $Kernel::OM->Get('Kernel::Output::HTML::Layout')->Block(
-            Name => 'MetaHttpEquivRefresh',
-            Data => \%Param,
-        );
-    }
+    $Kernel::OM->Get('Kernel::Output::HTML::Layout')->AddJSData(
+        Key   => 'Refresh',
+        Value => $Param{Refresh},
+    );
+
     return 1;
 }
 
