@@ -1,9 +1,9 @@
 # --
-# Copyright (C) 2001-2019 OTRS AG, https://otrs.com/
+# Copyright (C) 2001-2017 OTRS AG, http://otrs.com/
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
-# the enclosed file COPYING for license information (GPL). If you
-# did not receive this file, see https://www.gnu.org/licenses/gpl-3.0.txt.
+# the enclosed file COPYING for license information (AGPL). If you
+# did not receive this file, see http://www.gnu.org/licenses/agpl.txt.
 # --
 
 package Kernel::Output::HTML::Layout::Ticket;
@@ -20,15 +20,13 @@ our $ObjectManagerDisabled = 1;
 
 Kernel::Output::HTML::Layout::Ticket - all Ticket-related HTML functions
 
-=head1 SYNOPSIS
+=head1 DESCRIPTION
 
 All Ticket-related HTML functions
 
 =head1 PUBLIC INTERFACE
 
-=over 4
-
-=item AgentCustomerViewTable()
+=head2 AgentCustomerViewTable()
 
 =cut
 
@@ -115,7 +113,17 @@ sub AgentCustomerViewTable {
         }
     }
 
+    my $DynamicFieldConfigs = $Kernel::OM->Get('Kernel::System::DynamicField')->DynamicFieldListGet(
+        ObjectType => [ 'CustomerUser', 'CustomerCompany', ],
+    );
+
+    my %DynamicFieldLookup = map { $_->{Name} => $_ } @{$DynamicFieldConfigs};
+
+    # Get dynamic field object.
+    my $DynamicFieldBackendObject = $Kernel::OM->Get('Kernel::System::DynamicField::Backend');
+
     # build table
+    FIELD:
     for my $Field (@MapNew) {
         if ( $Field->[3] && $Field->[3] >= $ShownType && $Param{Data}->{ $Field->[0] } ) {
             my %Record = (
@@ -123,6 +131,39 @@ sub AgentCustomerViewTable {
                 Key   => $Field->[1],
                 Value => $Param{Data}->{ $Field->[0] },
             );
+
+            # render dynamic field values
+            if ( $Field->[5] eq 'dynamic_field' ) {
+                if ( !IsArrayRefWithData( $Record{Value} ) ) {
+                    $Record{Value} = [ $Record{Value} ];
+                }
+
+                my $DynamicFieldConfig = $DynamicFieldLookup{ $Field->[2] };
+
+                next FIELD if !$DynamicFieldConfig;
+
+                my @RenderedValues;
+                VALUE:
+                for my $Value ( @{ $Record{Value} } ) {
+                    my $RenderedValue = $DynamicFieldBackendObject->DisplayValueRender(
+                        DynamicFieldConfig => $DynamicFieldConfig,
+                        Value              => $Value,
+                        HTMLOutput         => 0,
+                        LayoutObject       => $Self,
+                    );
+
+                    next VALUE if !IsHashRefWithData($RenderedValue) || !defined $RenderedValue->{Value};
+
+                    # If there is configured show link in DF, save as map value.
+                    $Field->[6] = $RenderedValue->{Link} ? $RenderedValue->{Link} : $Field->[6];
+
+                    push @RenderedValues, $RenderedValue->{Value};
+                }
+
+                $Record{Value} = join ', ', @RenderedValues;
+                $Record{Key} = $DynamicFieldConfig->{Label};
+            }
+
             if (
                 $Field->[6]
                 && (
@@ -164,7 +205,7 @@ sub AgentCustomerViewTable {
                 my $CompanyValidID = $Param{Data}->{CustomerCompanyValidID};
 
                 if ($CompanyValidID) {
-                    my @ValidIDs       = $Kernel::OM->Get('Kernel::System::Valid')->ValidIDsGet();
+                    my @ValidIDs = $Kernel::OM->Get('Kernel::System::Valid')->ValidIDsGet();
                     my $CompanyIsValid = grep { $CompanyValidID == $_ } @ValidIDs;
 
                     if ( !$CompanyIsValid ) {
@@ -184,12 +225,13 @@ sub AgentCustomerViewTable {
                 my $EnableChat = 1;
                 my $ChatStartingAgentsGroup
                     = $ConfigObject->Get('ChatEngine::PermissionGroup::ChatStartingAgents') || 'users';
+                my $ChatStartingAgentsGroupPermission = $Kernel::OM->Get('Kernel::System::Group')->PermissionCheck(
+                    UserID    => $Self->{UserID},
+                    GroupName => $ChatStartingAgentsGroup,
+                    Type      => 'rw',
+                );
 
-                if (
-                    !defined $Self->{"UserIsGroup[$ChatStartingAgentsGroup]"}
-                    || $Self->{"UserIsGroup[$ChatStartingAgentsGroup]"} ne 'Yes'
-                    )
-                {
+                if ( !$ChatStartingAgentsGroupPermission ) {
                     $EnableChat = 0;
                 }
                 if (
@@ -204,13 +246,14 @@ sub AgentCustomerViewTable {
                     my $VideoChatEnabled = 0;
                     my $VideoChatAgentsGroup
                         = $ConfigObject->Get('ChatEngine::PermissionGroup::VideoChatAgents') || 'users';
+                    my $VideoChatAgentsGroupPermission = $Kernel::OM->Get('Kernel::System::Group')->PermissionCheck(
+                        UserID    => $Self->{UserID},
+                        GroupName => $VideoChatAgentsGroup,
+                        Type      => 'rw',
+                    );
 
                     # Enable the video chat feature if system is entitled and agent is a member of configured group.
-                    if (
-                        defined $Self->{"UserIsGroup[$VideoChatAgentsGroup]"}
-                        && $Self->{"UserIsGroup[$VideoChatAgentsGroup]"} eq 'Yes'
-                        )
-                    {
+                    if ($VideoChatAgentsGroupPermission) {
                         if ( $Kernel::OM->Get('Kernel::System::Main')
                             ->Require( 'Kernel::System::VideoChat', Silent => 1 ) )
                         {
@@ -379,7 +422,7 @@ sub AgentQueueListOption {
         # transform data from Hash in Array because of ordering in frontend by Queue name
         # it was a problem with name like '(some_queue)'
         # see bug#10621 http://bugs.otrs.org/show_bug.cgi?id=10621
-        my %QueueDataHash  = %{ $Param{Data} || {} };
+        my %QueueDataHash = %{ $Param{Data} || {} };
         my @QueueDataArray = map {
             {
                 Key   => $_,
@@ -390,8 +433,8 @@ sub AgentQueueListOption {
         # find index of first element in array @QueueDataArray for displaying in frontend
         # at the top should be element with ' $QueueDataArray[$_]->{Key} = 0' like "- Move -"
         # if such an element is found, it is moved to the top
-        my $MoveStr = $Self->{LanguageObject}->Translate('Move');
-        my $ValueOfQueueNoKey .= '- ' . $MoveStr . ' -';
+        my $MoveStr             = $Self->{LanguageObject}->Translate('Move');
+        my $ValueOfQueueNoKey   = '- ' . $MoveStr . ' -';
         my ($FirstElementIndex) = grep {
             $QueueDataArray[$_]->{Value} eq '-'
                 || $QueueDataArray[$_]->{Value} eq $ValueOfQueueNoKey
@@ -433,8 +476,8 @@ sub AgentQueueListOption {
     # add suffix for correct sorting
     my $KeyNoQueue;
     my $ValueNoQueue;
-    my $MoveStr = $Self->{LanguageObject}->Translate('Move');
-    my $ValueOfQueueNoKey .= "- " . $MoveStr . " -";
+    my $MoveStr           = $Self->{LanguageObject}->Translate('Move');
+    my $ValueOfQueueNoKey = "- " . $MoveStr . " -";
     DATA:
     for ( sort { $Data{$a} cmp $Data{$b} } keys %Data ) {
 
@@ -596,316 +639,6 @@ sub AgentQueueListOption {
     return $Param{MoveQueuesStrg};
 }
 
-=item ArticleQuote()
-
-get body and attach e. g. inline documents and/or attach all attachments to
-upload cache
-
-for forward or split, get body and attach all attachments
-
-    my $HTMLBody = $LayoutObject->ArticleQuote(
-        TicketID           => 123,
-        ArticleID          => 123,
-        FormID             => $Self->{FormID},
-        UploadCacheObject   => $Self->{UploadCacheObject},
-        AttachmentsInclude => 1,
-    );
-
-or just for including inline documents to upload cache
-
-    my $HTMLBody = $LayoutObject->ArticleQuote(
-        TicketID           => 123,
-        ArticleID          => 123,
-        FormID             => $Self->{FormID},
-        UploadCacheObject  => $Self->{UploadCacheObject},
-        AttachmentsInclude => 0,
-    );
-
-Both will also work without rich text (if $ConfigObject->Get('Frontend::RichText')
-is false), return param will be text/plain instead.
-
-=cut
-
-sub ArticleQuote {
-    my ( $Self, %Param ) = @_;
-
-    # check needed stuff
-    for (qw(TicketID ArticleID FormID UploadCacheObject)) {
-        if ( !$Param{$_} ) {
-            $Self->FatalError( Message => "Need $_!" );
-        }
-    }
-
-    # get needed objects
-    my $ConfigObject = $Kernel::OM->Get('Kernel::Config');
-    my $TicketObject = $Kernel::OM->Get('Kernel::System::Ticket');
-
-    # body preparation for plain text processing
-    if ( $ConfigObject->Get('Frontend::RichText') ) {
-
-        my $Body = '';
-
-        # check for html body
-        my @ArticleBox = $TicketObject->ArticleContentIndex(
-            TicketID                   => $Param{TicketID},
-            StripPlainBodyAsAttachment => 3,
-            UserID                     => $Self->{UserID},
-            DynamicFields              => 0,
-        );
-
-        my %NotInlineAttachments;
-        ARTICLE:
-        for my $ArticleTmp (@ArticleBox) {
-
-            # search for article to answer (reply article)
-            next ARTICLE if $ArticleTmp->{ArticleID} ne $Param{ArticleID};
-
-            # check if no html body exists
-            last ARTICLE if !$ArticleTmp->{AttachmentIDOfHTMLBody};
-
-            my %AttachmentHTML = $TicketObject->ArticleAttachment(
-                ArticleID => $ArticleTmp->{ArticleID},
-                FileID    => $ArticleTmp->{AttachmentIDOfHTMLBody},
-                UserID    => $Self->{UserID},
-            );
-            my $Charset = $AttachmentHTML{ContentType} || '';
-            $Charset =~ s/.+?charset=("|'|)(\w+)/$2/gi;
-            $Charset =~ s/"|'//g;
-            $Charset =~ s/(.+?);.*/$1/g;
-
-            # convert html body to correct charset
-            $Body = $Kernel::OM->Get('Kernel::System::Encode')->Convert(
-                Text  => $AttachmentHTML{Content},
-                From  => $Charset,
-                To    => $Self->{UserCharset},
-                Check => 1,
-            );
-
-            # get HTML utils object
-            my $HTMLUtilsObject = $Kernel::OM->Get('Kernel::System::HTMLUtils');
-
-            # add url quoting
-            $Body = $HTMLUtilsObject->LinkQuote(
-                String => $Body,
-            );
-
-            # strip head, body and meta elements
-            $Body = $HTMLUtilsObject->DocumentStrip(
-                String => $Body,
-            );
-
-            # display inline images if exists
-            my $SessionID = '';
-            if ( $Self->{SessionID} && !$Self->{SessionIDCookie} ) {
-                $SessionID = ';' . $Self->{SessionName} . '=' . $Self->{SessionID};
-            }
-            my $AttachmentLink = $Self->{Baselink}
-                . 'Action=PictureUpload'
-                . ';FormID='
-                . $Param{FormID}
-                . $SessionID
-                . ';ContentID=';
-
-            # search inline documents in body and add it to upload cache
-            my %Attachments = %{ $ArticleTmp->{Atms} };
-            my %AttachmentAlreadyUsed;
-            $Body =~ s{
-                (=|"|')cid:(.*?)("|'|>|\/>|\s)
-            }
-            {
-                my $Start= $1;
-                my $ContentID = $2;
-                my $End = $3;
-
-                # improve html quality
-                if ( $Start ne '"' && $Start ne '\'' ) {
-                    $Start .= '"';
-                }
-                if ( $End ne '"' && $End ne '\'' ) {
-                    $End = '"' . $End;
-                }
-
-                # find attachment to include
-                ATMCOUNT:
-                for my $AttachmentID ( sort keys %Attachments ) {
-
-                    if ( lc $Attachments{$AttachmentID}->{ContentID} ne lc "<$ContentID>" ) {
-                        next ATMCOUNT;
-                    }
-
-                    # get whole attachment
-                    my %AttachmentPicture = $TicketObject->ArticleAttachment(
-                        ArticleID => $Param{ArticleID},
-                        FileID    => $AttachmentID,
-                        UserID    => $Self->{UserID},
-                    );
-
-                    # content id cleanup
-                    $AttachmentPicture{ContentID} =~ s/^<//;
-                    $AttachmentPicture{ContentID} =~ s/>$//;
-
-                    # find cid, add attachment URL and remember, file is already uploaded
-                    $ContentID = $AttachmentLink . $Self->LinkEncode( $AttachmentPicture{ContentID} );
-
-                    # add to upload cache if not uploaded and remember
-                    if (!$AttachmentAlreadyUsed{$AttachmentID}) {
-
-                        # remember
-                        $AttachmentAlreadyUsed{$AttachmentID} = 1;
-
-                        # write attachment to upload cache
-                        $Param{UploadCacheObject}->FormIDAddFile(
-                            FormID      => $Param{FormID},
-                            Disposition => 'inline',
-                            %{ $Attachments{$AttachmentID} },
-                            %AttachmentPicture,
-                        );
-                    }
-                }
-
-                # return link
-                $Start . $ContentID . $End;
-            }egxi;
-
-            # find inline images using Content-Location instead of Content-ID
-            ATTACHMENT:
-            for my $AttachmentID ( sort keys %Attachments ) {
-
-                next ATTACHMENT if !$Attachments{$AttachmentID}->{ContentID};
-
-                # get whole attachment
-                my %AttachmentPicture = $TicketObject->ArticleAttachment(
-                    ArticleID => $Param{ArticleID},
-                    FileID    => $AttachmentID,
-                    UserID    => $Self->{UserID},
-                );
-
-                # content id cleanup
-                $AttachmentPicture{ContentID} =~ s/^<//;
-                $AttachmentPicture{ContentID} =~ s/>$//;
-
-                $Body =~ s{
-                    ("|')(\Q$AttachmentPicture{ContentID}\E)("|'|>|\/>|\s)
-                }
-                {
-                    my $Start= $1;
-                    my $ContentID = $2;
-                    my $End = $3;
-
-                    # find cid, add attachment URL and remember, file is already uploaded
-                    $ContentID = $AttachmentLink . $Self->LinkEncode( $AttachmentPicture{ContentID} );
-
-                    # add to upload cache if not uploaded and remember
-                    if (!$AttachmentAlreadyUsed{$AttachmentID}) {
-
-                        # remember
-                        $AttachmentAlreadyUsed{$AttachmentID} = 1;
-
-                        # write attachment to upload cache
-                        $Param{UploadCacheObject}->FormIDAddFile(
-                            FormID      => $Param{FormID},
-                            Disposition => 'inline',
-                            %{ $Attachments{$AttachmentID} },
-                            %AttachmentPicture,
-                        );
-                    }
-
-                    # return link
-                    $Start . $ContentID . $End;
-                }egxi;
-            }
-
-            # find not inline images
-            ATTACHMENT:
-            for my $AttachmentID ( sort keys %Attachments ) {
-                next ATTACHMENT if $AttachmentAlreadyUsed{$AttachmentID};
-                $NotInlineAttachments{$AttachmentID} = 1;
-            }
-
-            # do no more article
-            last ARTICLE;
-        }
-
-        # attach also other attachments on article forward
-        if ( $Body && $Param{AttachmentsInclude} ) {
-            for my $AttachmentID ( sort keys %NotInlineAttachments ) {
-                my %Attachment = $TicketObject->ArticleAttachment(
-                    ArticleID => $Param{ArticleID},
-                    FileID    => $AttachmentID,
-                    UserID    => $Self->{UserID},
-                );
-
-                # add attachment
-                $Param{UploadCacheObject}->FormIDAddFile(
-                    FormID => $Param{FormID},
-                    %Attachment,
-                    Disposition => 'attachment',
-                );
-            }
-        }
-        return $Body if $Body;
-    }
-
-    # as fallback use text body for quote
-    my %Article = $TicketObject->ArticleGet(
-        ArticleID     => $Param{ArticleID},
-        DynamicFields => 0,
-    );
-
-    # check if original content isn't text/plain or text/html, don't use it
-    if ( !$Article{ContentType} ) {
-        $Article{ContentType} = 'text/plain';
-    }
-
-    if ( $Article{ContentType} !~ /text\/(plain|html)/i ) {
-        $Article{Body}        = '-> no quotable message <-';
-        $Article{ContentType} = 'text/plain';
-    }
-    else {
-        $Article{Body} = $Self->WrapPlainText(
-            MaxCharacters => $ConfigObject->Get('Ticket::Frontend::TextAreaEmail') || 82,
-            PlainText     => $Article{Body},
-        );
-    }
-
-    # attach attachments
-    if ( $Param{AttachmentsInclude} ) {
-        my %ArticleIndex = $TicketObject->ArticleAttachmentIndex(
-            ArticleID                  => $Param{ArticleID},
-            UserID                     => $Self->{UserID},
-            StripPlainBodyAsAttachment => 3,
-            Article                    => \%Article,
-        );
-        for my $Index ( sort keys %ArticleIndex ) {
-            my %Attachment = $TicketObject->ArticleAttachment(
-                ArticleID => $Param{ArticleID},
-                FileID    => $Index,
-                UserID    => $Self->{UserID},
-            );
-
-            # add attachment
-            $Param{UploadCacheObject}->FormIDAddFile(
-                FormID => $Param{FormID},
-                %Attachment,
-                Disposition => 'attachment',
-            );
-        }
-    }
-
-    # return body as html
-    if ( $ConfigObject->Get('Frontend::RichText') ) {
-
-        $Article{Body} = $Self->Ascii2Html(
-            Text           => $Article{Body},
-            HTMLResultMode => 1,
-            LinkFeature    => 1,
-        );
-    }
-
-    # return body as plain text
-    return $Article{Body};
-}
-
 sub TicketListShow {
     my ( $Self, %Param ) = @_;
 
@@ -981,6 +714,14 @@ sub TicketListShow {
         }
     }
 
+    # get layout object
+    my $LayoutObject = $Kernel::OM->Get('Kernel::Output::HTML::Layout');
+
+    $LayoutObject->AddJSData(
+        Key   => 'View',
+        Value => $View,
+    );
+
     # load overview backend module
     if ( !$Kernel::OM->Get('Kernel::System::Main')->Require( $Backends->{$View}->{Module} ) ) {
         return $Env->{LayoutObject}->FatalError();
@@ -1030,7 +771,7 @@ sub TicketListShow {
     }
 
     # build nav bar
-    my $Limit   = $Param{Limit} || 20_000;
+    my $Limit = $Param{Limit} || 20_000;
     my %PageNav = $Self->PageNavBar(
         Limit     => $Limit,
         StartHit  => $StartHit,
@@ -1051,7 +792,6 @@ sub TicketListShow {
         Translation => 0,
         Data        => \%Data,
         Sort        => 'NumericValue',
-        Class       => 'Modernize',
     );
 
     # nav bar at the beginning of a overview
@@ -1067,6 +807,10 @@ sub TicketListShow {
             Name => 'OverviewNavBarPageBack',
             Data => \%Param,
         );
+        $LayoutObject->AddJSData(
+            Key   => 'Profile',
+            Value => $Param{Profile},
+        );
     }
 
     # filter selection
@@ -1370,14 +1114,12 @@ sub TicketMetaItems {
 
 1;
 
-=back
-
 =head1 TERMS AND CONDITIONS
 
-This software is part of the OTRS project (L<https://otrs.org/>).
+This software is part of the OTRS project (L<http://otrs.org/>).
 
 This software comes with ABSOLUTELY NO WARRANTY. For details, see
-the enclosed file COPYING for license information (GPL). If you
-did not receive this file, see L<https://www.gnu.org/licenses/gpl-3.0.txt>.
+the enclosed file COPYING for license information (AGPL). If you
+did not receive this file, see L<http://www.gnu.org/licenses/agpl.txt>.
 
 =cut
