@@ -1,13 +1,15 @@
 # --
-# Copyright (C) 2001-2018 OTRS AG, https://otrs.com/
+# Copyright (C) 2001-2017 OTRS AG, http://otrs.com/
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
-# the enclosed file COPYING for license information (GPL). If you
-# did not receive this file, see https://www.gnu.org/licenses/gpl-3.0.txt.
+# the enclosed file COPYING for license information (AGPL). If you
+# did not receive this file, see http://www.gnu.org/licenses/agpl.txt.
 # --
 
 package Kernel::Output::HTML::NavBar::CustomerTicketProcess;
 
+use parent 'Kernel::Output::HTML::Base';
+
 use strict;
 use warnings;
 
@@ -20,19 +22,6 @@ our @ObjectDependencies = (
     'Kernel::System::ProcessManagement::Process',
 );
 
-sub new {
-    my ( $Type, %Param ) = @_;
-
-    # allocate new hash for object
-    my $Self = {};
-    bless( $Self, $Type );
-
-    # get UserID param
-    $Self->{UserID} = $Param{UserID} || die "Got no UserID!";
-
-    return $Self;
-}
-
 sub Run {
     my ( $Self, %Param ) = @_;
 
@@ -40,14 +29,15 @@ sub Run {
     my $ConfigObject = $Kernel::OM->Get('Kernel::Config');
 
     # get process management configuration
-    my $FrontendModuleConfig = $ConfigObject->Get('CustomerFrontend::Module')->{CustomerTicketProcess};
+    my $FrontendModuleConfig     = $ConfigObject->Get('CustomerFrontend::Module')->{CustomerTicketProcess};
+    my $FrontendNavigationConfig = $ConfigObject->Get('CustomerFrontend::Navigation')->{CustomerTicketProcess};
 
     # check if the registration config is valid
     return if !IsHashRefWithData($FrontendModuleConfig);
+    return if !IsHashRefWithData($FrontendNavigationConfig);
+    return if !IsArrayRefWithData( $FrontendNavigationConfig->{'002-ProcessManagement'} );
 
-    return if !IsHashRefWithData( $FrontendModuleConfig->{NavBar}->[0] );
-
-    my $NameForID     = $FrontendModuleConfig->{NavBar}->[0]->{Name};
+    my $NameForID     = $FrontendNavigationConfig->{'002-ProcessManagement'}->[0]->{Name};
     my $NameForHidden = $NameForID;
     $NameForID =~ s/[ &;]//ig;
 
@@ -137,7 +127,7 @@ sub Run {
 
     # frontend module is enabled but there is no selectable process, then remove the menu entry
     my $NavBarName = $FrontendModuleConfig->{NavBarName};
-    my $Priority   = sprintf( "%07d", $FrontendModuleConfig->{NavBar}->[0]->{Prio} );
+    my $Priority = sprintf( '%07d', $FrontendNavigationConfig->{'002-ProcessManagement'}->[0]->{Prio} );
 
     my %Return = %{ $Param{NavBarModule}->{Sub} };
 
