@@ -1,5 +1,5 @@
 # --
-# Copyright (C) 2001-2018 OTRS AG, http://otrs.com/
+# Copyright (C) 2001-2017 OTRS AG, http://otrs.com/
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
 # the enclosed file COPYING for license information (AGPL). If you
@@ -16,15 +16,18 @@ use Kernel::Language qw(Translatable);
 
 our @ObjectDependencies = (
     'Kernel::Config',
+    'Kernel::Language',
     'Kernel::System::Log',
     'Kernel::Output::HTML::Layout',
     'Kernel::System::CustomerUser',
+    'Kernel::System::Group',
     'Kernel::System::User',
     'Kernel::System::JSON',
     'Kernel::System::DynamicField',
     'Kernel::System::Ticket::ColumnFilter',
     'Kernel::System::DynamicField::Backend',
     'Kernel::System::Ticket',
+    'Kernel::System::Ticket::Article',
     'Kernel::System::Main',
 );
 
@@ -41,8 +44,6 @@ sub new {
     # get config object
     my $ConfigObject = $Kernel::OM->Get('Kernel::Config');
 
-    $Self->{SmallViewColumnHeader} = $ConfigObject->Get('Ticket::Frontend::OverviewSmall')->{ColumnHeader};
-
     # set pref for columns key
     $Self->{PrefKeyColumns} = 'UserFilterColumnsEnabled' . '-' . $Self->{Action};
 
@@ -243,10 +244,15 @@ sub ActionRow {
             $BulkFeature = 1;
         }
         else {
+            my $GroupObject = $Kernel::OM->Get('Kernel::System::Group');
             GROUP:
             for my $Group (@Groups) {
-                next GROUP if !$LayoutObject->{"UserIsGroup[$Group]"};
-                if ( $LayoutObject->{"UserIsGroup[$Group]"} eq 'Yes' ) {
+                my $HasPermission = $GroupObject->PermissionCheck(
+                    UserID    => $Self->{UserID},
+                    GroupName => $Group,
+                    Type      => 'rw',
+                );
+                if ($HasPermission) {
                     $BulkFeature = 1;
                     last GROUP;
                 }
@@ -279,6 +285,9 @@ sub ActionRow {
         );
     }
 
+    my %ColumnTranslations;
+    my $LanguageObject = $Kernel::OM->Get('Kernel::Language');
+
     # add translations for the allocation lists for regular columns
     my $Columns = $Self->{Config}->{DefaultColumns} || $ConfigObject->Get('DefaultOverviewColumns') || {};
     if ( $Columns && IsHashRefWithData($Columns) ) {
@@ -306,21 +315,22 @@ sub ActionRow {
                 $TranslatedWord = Translatable('Pending till');
             }
             elsif ( $Column eq 'CustomerCompanyName' ) {
-                $TranslatedWord = Translatable('Customer Company Name');
+                $TranslatedWord = Translatable('Customer Name');
+            }
+            elsif ( $Column eq 'CustomerID' ) {
+                $TranslatedWord = Translatable('Customer ID');
+            }
+            elsif ( $Column eq 'CustomerName' ) {
+                $TranslatedWord = Translatable('Customer User Name');
             }
             elsif ( $Column eq 'CustomerUserID' ) {
                 $TranslatedWord = Translatable('Customer User ID');
             }
 
-            $LayoutObject->Block(
-                Name => 'ColumnTranslation',
-                Data => {
-                    ColumnName      => $Column,
-                    TranslateString => $TranslatedWord,
-                },
-            );
-            $LayoutObject->Block(
-                Name => 'ColumnTranslationSeparator',
+            # send data to JS
+            $LayoutObject->AddJSData(
+                Key   => 'Column' . $Column,
+                Value => $LanguageObject->Translate($TranslatedWord),
             );
         }
     }
@@ -342,19 +352,11 @@ sub ActionRow {
 
             $Counter++;
 
-            $LayoutObject->Block(
-                Name => 'ColumnTranslation',
-                Data => {
-                    ColumnName      => 'DynamicField_' . $DynamicField->{Name},
-                    TranslateString => $DynamicField->{Label},
-                },
+            # send data to JS
+            $LayoutObject->AddJSData(
+                Key   => 'ColumnDynamicField_' . $DynamicField->{Name},
+                Value => $LanguageObject->Translate( $DynamicField->{Label} ),
             );
-
-            if ( $Counter < scalar @{$ColumnsDynamicField} ) {
-                $LayoutObject->Block(
-                    Name => 'ColumnTranslationSeparator',
-                );
-            }
         }
     }
 
@@ -383,12 +385,11 @@ sub Run {
         $Self->{AvailableFilterableColumns} = {};    # disable all column filters
     }
 
-    # check needed stuff
-    for (qw(TicketIDs PageShown StartHit)) {
-        if ( !$Param{$_} ) {
+    for my $Item (qw(TicketIDs PageShown StartHit)) {
+        if ( !$Param{$Item} ) {
             $Kernel::OM->Get('Kernel::System::Log')->Log(
                 Priority => 'error',
-                Message  => "Need $_!"
+                Message  => "Need $Item!",
             );
             return;
         }
@@ -409,10 +410,15 @@ sub Run {
             $BulkFeature = 1;
         }
         else {
+            my $GroupObject = $Kernel::OM->Get('Kernel::System::Group');
             GROUP:
             for my $Group (@Groups) {
-                next GROUP if !$LayoutObject->{"UserIsGroup[$Group]"};
-                if ( $LayoutObject->{"UserIsGroup[$Group]"} eq 'Yes' ) {
+                my $HasPermission = $GroupObject->PermissionCheck(
+                    UserID    => $Self->{UserID},
+                    GroupName => $Group,
+                    Type      => 'rw',
+                );
+                if ($HasPermission) {
                     $BulkFeature = 1;
                     last GROUP;
                 }
@@ -422,28 +428,69 @@ sub Run {
 
     my $Counter = 0;
     my @ArticleBox;
+
+    my $TicketObject  = $Kernel::OM->Get('Kernel::System::Ticket');
+    my $ArticleObject = $Kernel::OM->Get('Kernel::System::Ticket::Article');
+
     for my $TicketID ( @{ $Param{TicketIDs} } ) {
         $Counter++;
         if ( $Counter >= $Param{StartHit} && $Counter < ( $Param{PageShown} + $Param{StartHit} ) ) {
 
-            # get ticket object
-            my $TicketObject = $Kernel::OM->Get('Kernel::System::Ticket');
-
-            # get last customer article
-            my %Article = $TicketObject->ArticleLastCustomerArticle(
-                TicketID      => $TicketID,
-                Extended      => 1,
-                DynamicFields => 0,
+            # Get last customer article.
+            my @Articles = $ArticleObject->ArticleList(
+                TicketID   => $TicketID,
+                SenderType => 'customer',
+                OnlyLast   => 1,
             );
 
-            # get ticket data
+            # If the ticket has no customer article, get the last agent article.
+            if ( !@Articles ) {
+                @Articles = $ArticleObject->ArticleList(
+                    TicketID   => $TicketID,
+                    SenderType => 'agent',
+                    OnlyLast   => 1,
+                );
+            }
+
+            # Finally, if everything failed, get latest article.
+            if ( !@Articles ) {
+                @Articles = $ArticleObject->ArticleList(
+                    TicketID => $TicketID,
+                    OnlyLast => 1,
+                );
+            }
+
+            my %Article;
+            for my $Article (@Articles) {
+                %Article = $ArticleObject->BackendForArticle( %{$Article} )->ArticleGet(
+                    %{$Article},
+                    DynamicFields => 0,
+                );
+            }
+
+            # Get ticket data.
             my %Ticket = $TicketObject->TicketGet(
                 TicketID      => $TicketID,
                 Extended      => 1,
                 DynamicFields => 0,
             );
 
-            # Fallback for tickets without articles: get at least basic ticket data
+            %Article = ( %Article, %Ticket );
+
+            # Get channel specific fields.
+            if ( $Article{ArticleID} ) {
+                my %ArticleFields = $LayoutObject->ArticleFields(
+                    TicketID  => $TicketID,
+                    ArticleID => $Article{ArticleID},
+                );
+                FIELD:
+                for my $FieldKey (qw(Sender Subject)) {
+                    next FIELD if !defined $ArticleFields{$FieldKey}->{Value};
+                    $Article{$FieldKey} = $ArticleFields{$FieldKey}->{Realname} // $ArticleFields{$FieldKey}->{Value};
+                }
+            }
+
+            # Fallback for tickets without articles: get at least basic ticket data.
             if ( !%Article ) {
                 %Article = %Ticket;
                 if ( !$Article{Title} ) {
@@ -457,21 +504,6 @@ sub Run {
             # show ticket create time in small view
             $Article{Created} = $Ticket{Created};
 
-            # prepare a "long" version of the subject to show in the title attribute. We don't take
-            # the whole string (which could be VERY long) to avoid polluting the DOM and having too
-            # much data to be transferred on large ticket lists
-            $Article{SubjectLong} = $TicketObject->TicketSubjectClean(
-                TicketNumber => $Article{TicketNumber},
-                Subject      => $Article{Subject} || '',
-                Size         => 500,
-            );
-
-            # prepare subject
-            $Article{Subject} = $TicketObject->TicketSubjectClean(
-                TicketNumber => $Article{TicketNumber},
-                Subject      => $Article{Subject} || '',
-            );
-
             # create human age
             $Article{Age} = $LayoutObject->CustomerAge(
                 Age   => $Article{Age},
@@ -601,6 +633,12 @@ sub Run {
         OriginalTicketIDs => $Param{OriginalTicketIDs},
     );
 
+    # send data to JS
+    $LayoutObject->AddJSData(
+        Key   => 'LinkPage',
+        Value => $Param{LinkPage},
+    );
+
     $LayoutObject->Block(
         Name => 'DocumentContent',
         Data => \%Param,
@@ -615,7 +653,6 @@ sub Run {
         Owner        => 1,
         Responsible  => 1,
         CustomerID   => 1,
-        Title        => 1,
     );
 
     # get dynamic field backend object
@@ -739,19 +776,8 @@ sub Run {
 
                 # translate the column name to write it in the current language
                 my $TranslatedWord;
-
-                if ( $Column eq 'Title' ) {
-
-                    $TranslatedWord = $LayoutObject->{LanguageObject}->Translate('From') . ' / ';
-
-                    if ( $Self->{SmallViewColumnHeader} eq 'LastCustomerSubject' ) {
-                        $TranslatedWord
-                            .= $LayoutObject->{LanguageObject}->Translate('Subject');
-                    }
-                    elsif ( $Self->{SmallViewColumnHeader} eq 'TicketTitle' ) {
-                        $TranslatedWord
-                            .= $LayoutObject->{LanguageObject}->Translate('Title');
-                    }
+                if ( $Column eq 'CustomerID' ) {
+                    $TranslatedWord = $LayoutObject->{LanguageObject}->Translate('Customer ID');
                 }
                 else {
                     $TranslatedWord = $LayoutObject->{LanguageObject}->Translate($Column);
@@ -827,10 +853,16 @@ sub Run {
                         $LayoutObject->Block(
                             Name =>
                                 'ContentLargeTicketGenericHeaderColumnFilterLinkCustomerIDSearch',
-                            Data => {
-                                minQueryLength      => 2,
-                                queryDelay          => 100,
-                                maxResultsDisplayed => 20,
+                            Data => {},
+                        );
+
+                        # send data to JS
+                        $LayoutObject->AddJSData(
+                            Key   => 'CustomerIDAutocomplete',
+                            Value => {
+                                'QueryDelay'          => 100,
+                                'MaxResultsDisplayed' => 20,
+                                'MinQueryLength'      => 2,
                             },
                         );
                     }
@@ -838,10 +870,16 @@ sub Run {
 
                         $LayoutObject->Block(
                             Name => 'ContentLargeTicketGenericHeaderColumnFilterLinkUserSearch',
-                            Data => {
-                                minQueryLength      => 2,
-                                queryDelay          => 100,
-                                maxResultsDisplayed => 20,
+                            Data => {},
+                        );
+
+                        # send data to JS
+                        $LayoutObject->AddJSData(
+                            Key   => 'UserAutocomplete',
+                            Value => {
+                                'QueryDelay'          => 100,
+                                'MaxResultsDisplayed' => 20,
+                                'MinQueryLength'      => 2,
                             },
                         );
                     }
@@ -939,7 +977,10 @@ sub Run {
                     $TranslatedWord = $LayoutObject->{LanguageObject}->Translate('Pending till');
                 }
                 elsif ( $Column eq 'CustomerCompanyName' ) {
-                    $TranslatedWord = $LayoutObject->{LanguageObject}->Translate('Customer Company Name');
+                    $TranslatedWord = $LayoutObject->{LanguageObject}->Translate('Customer Name');
+                }
+                elsif ( $Column eq 'CustomerName' ) {
+                    $TranslatedWord = $LayoutObject->{LanguageObject}->Translate('Customer User Name');
                 }
                 elsif ( $Column eq 'CustomerUserID' ) {
                     $TranslatedWord = $LayoutObject->{LanguageObject}->Translate('Customer User ID');
@@ -1031,10 +1072,16 @@ sub Run {
                         $LayoutObject->Block(
                             Name =>
                                 'ContentLargeTicketGenericHeaderColumnFilterLinkCustomerUserSearch',
-                            Data => {
-                                minQueryLength      => 2,
-                                queryDelay          => 100,
-                                maxResultsDisplayed => 20,
+                            Data => {},
+                        );
+
+                        # send data to JS
+                        $LayoutObject->AddJSData(
+                            Key   => 'CustomerUserAutocomplete',
+                            Value => {
+                                'QueryDelay'          => 100,
+                                'MaxResultsDisplayed' => 20,
+                                'MinQueryLength'      => 2,
                             },
                         );
                     }
@@ -1325,6 +1372,8 @@ sub Run {
         $LayoutObject->Block( Name => 'NoTicketFound' );
     }
 
+    my %ActionRowTickets;
+
     for my $ArticleRef (@ArticleBox) {
 
         # get last customer article
@@ -1332,13 +1381,15 @@ sub Run {
 
         # escalation human times
         if ( $Article{EscalationTime} ) {
-            $Article{EscalationTimeHuman} = $LayoutObject->CustomerAgeInHours(
-                Age   => $Article{EscalationTime},
-                Space => ' ',
+            $Article{EscalationTimeHuman} = $LayoutObject->CustomerAge(
+                Age                => $Article{EscalationTime},
+                TimeShowAlwaysLong => 1,
+                Space              => ' ',
             );
-            $Article{EscalationTimeWorkingTime} = $LayoutObject->CustomerAgeInHours(
-                Age   => $Article{EscalationTimeWorkingTime},
-                Space => ' ',
+            $Article{EscalationTimeWorkingTime} = $LayoutObject->CustomerAge(
+                Age                => $Article{EscalationTimeWorkingTime},
+                TimeShowAlwaysLong => 1,
+                Space              => ' ',
             );
         }
 
@@ -1433,35 +1484,18 @@ sub Run {
                 );
 
                 if ( $SpecialColumns{$TicketColumn} ) {
-                    if ( $TicketColumn eq 'Title' ) {
+                    $LayoutObject->Block(
+                        Name => 'Record' . $TicketColumn,
+                        Data => { %Article, %UserInfo },
+                    );
 
-                        # check if last customer subject or ticket title should be shown
-                        if ( $Self->{SmallViewColumnHeader} eq 'LastCustomerSubject' ) {
-                            $LayoutObject->Block(
-                                Name => 'RecordLastCustomerSubject',
-                                Data => { %Article, %UserInfo },
-                            );
-                        }
-                        elsif ( $Self->{SmallViewColumnHeader} eq 'TicketTitle' ) {
-                            $LayoutObject->Block(
-                                Name => 'RecordTicketTitle',
-                                Data => { %Article, %UserInfo },
-                            );
-                        }
-                    }
-                    else {
-                        $LayoutObject->Block(
-                            Name => 'Record' . $TicketColumn,
-                            Data => { %Article, %UserInfo },
-                        );
-                    }
                     next TICKETCOLUMN;
                 }
 
                 if ( $TicketColumn eq 'CreatedBy' ) {
 
                     my %TicketCreatedByInfo = $UserObject->GetUserData(
-                        UserID => $Article{CreatedBy},
+                        UserID => $Article{CreateBy},
                     );
 
                     $LayoutObject->Block(
@@ -1477,13 +1511,15 @@ sub Run {
                     $EscalationData{EscalationTime}            = $Article{EscalationTime};
                     $EscalationData{EscalationDestinationDate} = $Article{EscalationDestinationDate};
 
-                    $EscalationData{EscalationTimeHuman} = $LayoutObject->CustomerAgeInHours(
-                        Age   => $EscalationData{EscalationTime},
-                        Space => ' ',
+                    $EscalationData{EscalationTimeHuman} = $LayoutObject->CustomerAge(
+                        Age                => $EscalationData{EscalationTime},
+                        TimeShowAlwaysLong => 1,
+                        Space              => ' ',
                     );
-                    $EscalationData{EscalationTimeWorkingTime} = $LayoutObject->CustomerAgeInHours(
-                        Age   => $EscalationData{EscalationTimeWorkingTime},
-                        Space => ' ',
+                    $EscalationData{EscalationTimeWorkingTime} = $LayoutObject->CustomerAge(
+                        Age                => $EscalationData{EscalationTimeWorkingTime},
+                        TimeShowAlwaysLong => 1,
+                        Space              => ' ',
                     );
                     if (
                         defined $Article{EscalationTime}
@@ -1503,28 +1539,21 @@ sub Run {
                 my $CSSClass  = '';
                 if ( $TicketColumn eq 'EscalationSolutionTime' ) {
                     $BlockType = 'Escalation';
-
-                    # Article SolutionTime is taken into consideration only when there is EscalationSolutionTime.
-                    # See bug#12912 (https://bugs.otrs.org/show_bug.cgi?id=12912).
-                    my $SolutionTime = 0;
-                    if ( $Article{EscalationSolutionTime} ) {
-                        $SolutionTime = $Article{SolutionTime};
-
-                        if ( $SolutionTime < 60 * 60 * 1 ) {
-                            $CSSClass = 'Warning';
-                        }
-                    }
-
-                    $DataValue = $LayoutObject->CustomerAgeInHours(
-                        Age   => $SolutionTime,
-                        Space => ' ',
+                    $DataValue = $LayoutObject->CustomerAge(
+                        Age => $Article{SolutionTime} || 0,
+                        TimeShowAlwaysLong => 1,
+                        Space              => ' ',
                     );
+                    if ( defined $Article{SolutionTime} && $Article{SolutionTime} < 60 * 60 * 1 ) {
+                        $CSSClass = 'Warning';
+                    }
                 }
                 elsif ( $TicketColumn eq 'EscalationResponseTime' ) {
                     $BlockType = 'Escalation';
-                    $DataValue = $LayoutObject->CustomerAgeInHours(
+                    $DataValue = $LayoutObject->CustomerAge(
                         Age => $Article{FirstResponseTime} || 0,
-                        Space => ' ',
+                        TimeShowAlwaysLong => 1,
+                        Space              => ' ',
                     );
                     if (
                         defined $Article{FirstResponseTime}
@@ -1536,9 +1565,10 @@ sub Run {
                 }
                 elsif ( $TicketColumn eq 'EscalationUpdateTime' ) {
                     $BlockType = 'Escalation';
-                    $DataValue = $LayoutObject->CustomerAgeInHours(
+                    $DataValue = $LayoutObject->CustomerAge(
                         Age => $Article{UpdateTime} || 0,
-                        Space => ' ',
+                        TimeShowAlwaysLong => 1,
+                        Space              => ' ',
                     );
                     if ( defined $Article{UpdateTime} && $Article{UpdateTime} < 60 * 60 * 1 ) {
                         $CSSClass = 'Warning';
@@ -1567,6 +1597,14 @@ sub Run {
                     $BlockType = 'Time';
                     $DataValue = $Article{$TicketColumn} || $UserInfo{$TicketColumn};
                 }
+                elsif ( $TicketColumn eq 'Responsible' ) {
+
+                    my %ResponsibleInfo = $UserObject->GetUserData(
+                        UserID => $Article{ResponsibleID},
+                    );
+
+                    $DataValue = $ResponsibleInfo{'UserFullname'};
+                }
                 else {
                     $DataValue = $Article{$TicketColumn}
                         || $UserInfo{$TicketColumn}
@@ -1677,20 +1715,24 @@ sub Run {
         # add action items as js
         if ( $Article{ActionItems} ) {
 
-            $LayoutObject->Block(
-                Name => 'DocumentReadyActionRowAdd',
-                Data => {
-                    TicketID => $Article{TicketID},
-                    Data     => $Article{ActionItems},
-                },
-            );
+            # replace TT directives from string with values
+            for my $ActionItem ( @{ $Article{ActionItems} } ) {
+                $ActionItem->{Link} = $LayoutObject->Output(
+                    Template => $ActionItem->{Link},
+                    Data     => {
+                        TicketID => $Article{TicketID},
+                    },
+                );
+            }
+
+            $ActionRowTickets{ $Article{TicketID} } = $LayoutObject->JSONEncode( Data => $Article{ActionItems} );
         }
     }
 
-    # init for table control
-    $LayoutObject->Block(
-        Name => 'DocumentReadyStart',
-        Data => \%Param,
+    # send data to JS
+    $LayoutObject->AddJSData(
+        Key   => 'ActionRowTickets',
+        Value => \%ActionRowTickets,
     );
 
     # set column filter form, to correctly fill the column filters is necessary to pass each
@@ -1901,23 +1943,21 @@ sub FilterContent {
     return $ColumnFilterJSON;
 }
 
-=over
+# =head2 _ColumnFilterJSON()
 
-=item _ColumnFilterJSON()
+#     creates a JSON select filter for column header
 
-    creates a JSON select filter for column header
+#     my $ColumnFilterJSON = $TicketOverviewSmallObject->_ColumnFilterJSON(
+#         ColumnName => 'Queue',
+#         Label      => 'Queue',
+#         ColumnValues => {
+#             1 => 'PostMaster',
+#             2 => 'Junk',
+#         },
+#         SelectedValue '1',
+#     );
 
-    my $ColumnFilterJSON = $TicketOverviewSmallObject->_ColumnFilterJSON(
-        ColumnName => 'Queue',
-        Label      => 'Queue',
-        ColumnValues => {
-            1 => 'PostMaster',
-            2 => 'Junk',
-        },
-        SelectedValue '1',
-    );
-
-=cut
+# =cut
 
 sub _ColumnFilterJSON {
     my ( $Self, %Param ) = @_;
@@ -1958,7 +1998,7 @@ sub _ColumnFilterJSON {
         for my $ValueKey ( sort { lc $Values{$a} cmp lc $Values{$b} } keys %Values ) {
             push @{$Data}, {
                 Key   => $ValueKey,
-                Value => $Values{$ValueKey}
+                Value => $Values{$ValueKey},
             };
         }
     }
@@ -2005,7 +2045,9 @@ sub _DefaultColumnSort {
         EscalationSolutionTime => 114,
         EscalationResponseTime => 115,
         EscalationUpdateTime   => 116,
-        Title                  => 120,
+        Sender                 => 120,
+        Title                  => 122,
+        Subject                => 124,
         State                  => 130,
         Lock                   => 140,
         Queue                  => 150,
@@ -2041,5 +2083,3 @@ sub _DefaultColumnSort {
 }
 
 1;
-
-=back
