@@ -1,9 +1,9 @@
 # --
-# Copyright (C) 2001-2018 OTRS AG, https://otrs.com/
+# Copyright (C) 2001-2017 OTRS AG, http://otrs.com/
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
-# the enclosed file COPYING for license information (GPL). If you
-# did not receive this file, see https://www.gnu.org/licenses/gpl-3.0.txt.
+# the enclosed file COPYING for license information (AGPL). If you
+# did not receive this file, see http://www.gnu.org/licenses/agpl.txt.
 # --
 
 package Kernel::System::Cache::FileStorable;
@@ -12,7 +12,6 @@ use strict;
 use warnings;
 
 use POSIX;
-use Storable qw();
 use Digest::MD5 qw();
 use File::Path qw();
 use File::Find qw();
@@ -22,6 +21,7 @@ our @ObjectDependencies = (
     'Kernel::System::Encode',
     'Kernel::System::Log',
     'Kernel::System::Main',
+    'Kernel::System::Storable',
 );
 
 sub new {
@@ -71,11 +71,11 @@ sub Set {
         }
     }
 
-    my $Dump = Storable::nfreeze(
-        {
+    my $Dump = $Kernel::OM->Get('Kernel::System::Storable')->Serialize(
+        Data => {
             TTL   => time() + $Param{TTL},
             Value => $Param{Value},
-        }
+        },
     );
 
     my ( $Filename, $CacheDirectory ) = $Self->_GetFilenameAndCacheDirectory(%Param);
@@ -138,7 +138,11 @@ sub Get {
     return if !${$Content};
 
     # read data structure back from file dump, use block eval for safety reasons
-    my $Storage = eval { Storable::thaw( ${$Content} ) };
+    my $Storage = eval {
+        $Kernel::OM->Get('Kernel::System::Storable')->Deserialize(
+            Data => ${$Content}
+        );
+    };
     if ( ref $Storage ne 'HASH' || $Storage->{TTL} < time() ) {
         $Self->Delete(%Param);
         return;
@@ -208,7 +212,9 @@ sub CleanUp {
             );
 
             if ( ref $Content eq 'SCALAR' ) {
-                my $Storage = eval { Storable::thaw( ${$Content} ); };
+                my $Storage = eval {
+                    $Kernel::OM->Get('Kernel::System::Storable')->Deserialize( Data => ${$Content} );
+                };
                 return if ( ref $Storage eq 'HASH' && $Storage->{TTL} > time() );
             }
         }
@@ -257,7 +263,7 @@ sub _GetFilenameAndCacheDirectory {
     my $CacheDirectory = $Self->{CacheDirectory} . '/' . $Param{Type};
 
     for my $Level ( 1 .. $Self->{'Cache::SubdirLevels'} ) {
-        $CacheDirectory .= '/' . substr( $Filename, $Level - 1, 1 );
+        $CacheDirectory .= '/' . substr( $Filename, $Level - 1, 1 )
     }
 
     return $Filename, $CacheDirectory;
