@@ -1,9 +1,9 @@
 # --
-# Copyright (C) 2001-2020 OTRS AG, https://otrs.com/
+# Copyright (C) 2001-2017 OTRS AG, http://otrs.com/
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
-# the enclosed file COPYING for license information (GPL). If you
-# did not receive this file, see https://www.gnu.org/licenses/gpl-3.0.txt.
+# the enclosed file COPYING for license information (AGPL). If you
+# did not receive this file, see http://www.gnu.org/licenses/agpl.txt.
 # --
 
 package Kernel::System::Console::Command::Dev::Code::Generate::ConsoleCommand;
@@ -15,7 +15,7 @@ use warnings;
 use File::Path     ();
 use File::Basename ();
 
-use base qw(Kernel::System::Console::BaseCommand);
+use parent qw(Kernel::System::Console::BaseCommand);
 
 our @ObjectDependencies = (
     'Kernel::Config',
@@ -41,6 +41,7 @@ sub Configure {
         Required    => 1,
         ValueRegex  => qr/.*/smx,
     );
+    return;
 }
 
 sub PreRun {
@@ -66,9 +67,6 @@ sub Run {
         $TargetHome = $ModuleDirectory;
     }
 
-    # Keep comment lines in files also in the generated output.
-    local $ENV{TEMPLATE_KEEP_COMMENTS} = 1;
-
     # create perl module file
     my $CommandPathPM = $CommandName . ".pm";
     $CommandPathPM =~ s{::}{/}smxg;
