@@ -1,6 +1,5 @@
 # --
-# Kernel/System/DB/mysql.pm - mysql database backend
-# Copyright (C) 2001-2015 OTRS AG, http://otrs.com/
+# Copyright (C) 2001-2017 OTRS AG, http://otrs.com/
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
 # the enclosed file COPYING for license information (AGPL). If you
@@ -12,6 +11,16 @@ package Kernel::System::DB::mysql;
 use strict;
 use warnings;
 
+use Encode ();
+
+our @ObjectDependencies = (
+    'Kernel::Config',
+    'Kernel::System::DateTime',
+    'Kernel::System::Encode',
+    'Kernel::System::Log',
+    'Kernel::System::Main',
+);
+
 sub new {
     my ( $Type, %Param ) = @_;
 
@@ -33,9 +42,22 @@ sub LoadPreferences {
     $Self->{'DB::QuoteSemicolon'}       = '\\';
     $Self->{'DB::QuoteUnderscoreStart'} = '\\';
     $Self->{'DB::QuoteUnderscoreEnd'}   = '';
-    $Self->{'DB::CaseInsensitive'}      = 1;
+    $Self->{'DB::CaseSensitive'}        = 0;
     $Self->{'DB::LikeEscapeString'}     = '';
 
+    # mysql needs to proprocess the data to fix UTF8 issues
+    $Self->{'DB::PreProcessSQL'}      = 1;
+    $Self->{'DB::PreProcessBindData'} = 1;
+
+    # how to determine server version
+    # version can have package prefix, we need to extract that
+    # example of VERSION() output: '5.5.32-0ubuntu0.12.04.1'
+    # if VERSION() contains 'MariaDB', add MariaDB, otherwise MySQL.
+    $Self->{'DB::Version'}
+        = "SELECT CONCAT( IF (INSTR( VERSION(),'MariaDB'),'MariaDB ','MySQL '), SUBSTRING_INDEX(VERSION(),'-',1))";
+
+    $Self->{'DB::ListTables'} = 'SHOW TABLES';
+
     # DBI/DBD::mysql attributes
     # disable automatic reconnects as they do not execute DB::Connect, which will
     # cause charset problems
@@ -56,14 +78,57 @@ sub LoadPreferences {
     #$Self->{'DB::ShellConnect'} = '';
 
     # init sql setting on db connect
-    if ( !$Self->{ConfigObject}->Get('Database::ShellOutput') )
-    {
+    if ( !$Kernel::OM->Get('Kernel::Config')->Get('Database::ShellOutput') ) {
         $Self->{'DB::Connect'} = 'SET NAMES utf8';
     }
 
     return 1;
 }
 
+sub PreProcessSQL {
+    my ( $Self, $SQLRef ) = @_;
+    $Self->_FixMysqlUTF8($SQLRef);
+    $Kernel::OM->Get('Kernel::System::Encode')->EncodeOutput($SQLRef);
+    return;
+}
+
+sub PreProcessBindData {
+    my ( $Self, $BindRef ) = @_;
+
+    my $Size = scalar @{ $BindRef // [] };
+
+    my $EncodeObject = $Kernel::OM->Get('Kernel::System::Encode');
+
+    for ( my $I = 0; $I < $Size; $I++ ) {
+
+        $Self->_FixMysqlUTF8( \$BindRef->[$I] );
+
+        # DBD::mysql 4.042+ requires data to be octets, so we encode the data on our own.
+        #   The mysql_enable_utf8 flag seems to be unusable because it treats ALL data as UTF8 unless
+        #   it has a custom bind data type like SQL_BLOB.
+        #
+        #   See also https://bugs.otrs.org/show_bug.cgi?id=12677.
+        $EncodeObject->EncodeOutput( \$BindRef->[$I] );
+    }
+    return;
+}
+
+# Replace any unicode characters that need more than three bytes in UTF8
+#   with the unicode replacement character. MySQL's utf8 encoding only
+#   supports three bytes. In future we might want to use utf8mb4 (supported
+#   since 5.5.3+), but that will lead to problems with key sizes on mysql.
+# See also http://mathiasbynens.be/notes/mysql-utf8mb4.
+sub _FixMysqlUTF8 {
+    my ( $Self, $StringRef ) = @_;
+
+    return if !$$StringRef;
+    return if !Encode::is_utf8($$StringRef);
+
+    $$StringRef =~ s/([\x{10000}-\x{10FFFF}])/"\x{FFFD}"/eg;
+
+    return;
+}
+
 sub Quote {
     my ( $Self, $Text, $Type ) = @_;
 
@@ -92,7 +157,7 @@ sub DatabaseCreate {
 
     # check needed stuff
     if ( !$Param{Name} ) {
-        $Self->{LogObject}->Log(
+        $Kernel::OM->Get('Kernel::System::Log')->Log(
             Priority => 'error',
             Message  => 'Need Name!'
         );
@@ -108,7 +173,7 @@ sub DatabaseDrop {
 
     # check needed stuff
     if ( !$Param{Name} ) {
-        $Self->{LogObject}->Log(
+        $Kernel::OM->Get('Kernel::System::Log')->Log(
             Priority => 'error',
             Message  => 'Need Name!'
         );
@@ -122,6 +187,9 @@ sub DatabaseDrop {
 sub TableCreate {
     my ( $Self, @Param ) = @_;
 
+    # get config object
+    my $ConfigObject = $Kernel::OM->Get('Kernel::Config');
+
     my $SQLStart     = '';
     my $SQLEnd       = '';
     my $SQL          = '';
@@ -142,7 +210,7 @@ sub TableCreate {
             && $Tag->{TagType} eq 'Start'
             )
         {
-            if ( $Self->{ConfigObject}->Get('Database::ShellOutput') ) {
+            if ( $ConfigObject->Get('Database::ShellOutput') ) {
                 $SQLStart .= $Self->{'DB::Comment'}
                     . "----------------------------------------------------------\n";
                 $SQLStart .= $Self->{'DB::Comment'} . " create table $Tag->{Name}\n";
@@ -268,35 +336,17 @@ sub TableCreate {
     $SQL .= "\n";
     push @Return, $SQLStart . $SQL . $SQLEnd;
 
-    # add indexs
-    #    for my $Name (sort keys %Index) {
-    #        push (@Return, $Self->IndexCreate(
-    #            TableName => $TableName,
-    #            Name => $Name,
-    #            Data => $Index{$Name},
-    #        ));
-    #    }
-    # add uniq
-    #    for my $Name (sort keys %Uniq) {
-    #        push (@Return, $Self->UniqueCreate(
-    #            TableName => $TableName,
-    #            Name => $Name,
-    #            Data => $Uniq{$Name},
-    #        ));
-    #    }
     # add foreign keys
     for my $ForeignKey ( sort keys %Foreign ) {
         my @Array = @{ $Foreign{$ForeignKey} };
         for ( 0 .. $#Array ) {
-            push(
-                @{ $Self->{Post} },
+            push @{ $Self->{Post} },
                 $Self->ForeignKeyCreate(
-                    LocalTableName   => $TableName,
-                    Local            => $Array[$_]->{Local},
-                    ForeignTableName => $ForeignKey,
-                    Foreign          => $Array[$_]->{Foreign},
-                ),
-            );
+                LocalTableName   => $TableName,
+                Local            => $Array[$_]->{Local},
+                ForeignTableName => $ForeignKey,
+                Foreign          => $Array[$_]->{Foreign},
+                );
         }
     }
     return @Return;
@@ -305,10 +355,13 @@ sub TableCreate {
 sub TableDrop {
     my ( $Self, @Param ) = @_;
 
+    # get config object
+    my $ConfigObject = $Kernel::OM->Get('Kernel::Config');
+
     my $SQL = '';
     for my $Tag (@Param) {
         if ( $Tag->{Tag} eq 'Table' && $Tag->{TagType} eq 'Start' ) {
-            if ( $Self->{ConfigObject}->Get('Database::ShellOutput') ) {
+            if ( $ConfigObject->Get('Database::ShellOutput') ) {
                 $SQL .= $Self->{'DB::Comment'}
                     . "----------------------------------------------------------\n";
                 $SQL .= $Self->{'DB::Comment'} . " drop table $Tag->{Name}\n";
@@ -325,6 +378,9 @@ sub TableDrop {
 sub TableAlter {
     my ( $Self, @Param ) = @_;
 
+    # get config object
+    my $ConfigObject = $Kernel::OM->Get('Kernel::Config');
+
     my $SQLStart      = '';
     my @SQL           = ();
     my @Index         = ();
@@ -337,7 +393,7 @@ sub TableAlter {
 
         if ( $Tag->{Tag} eq 'TableAlter' && $Tag->{TagType} eq 'Start' ) {
             $Table = $Tag->{Name} || $Tag->{NameNew};
-            if ( $Self->{ConfigObject}->Get('Database::ShellOutput') ) {
+            if ( $ConfigObject->Get('Database::ShellOutput') ) {
                 $SQLStart .= $Self->{'DB::Comment'}
                     . "----------------------------------------------------------\n";
                 $SQLStart .= $Self->{'DB::Comment'} . " alter table $Table\n";
@@ -403,9 +459,9 @@ sub TableAlter {
             # normal data type
             push @SQL, $SQLStart . " CHANGE $Tag->{NameOld} $Tag->{NameNew} $Tag->{Type} NULL";
 
-            # remove possible default (not on TEXT/BLOB/LONGBLOB type, not supported by mysql)
+            # set default as NULL (not on TEXT/BLOB/LONGBLOB type, not supported by mysql)
             if ( $Tag->{Type} !~ /^(TEXT|MEDIUMTEXT|BLOB|LONGBLOB)$/i ) {
-                push @SQL, "ALTER TABLE $Table ALTER $Tag->{NameNew} DROP DEFAULT";
+                push @SQL, "ALTER TABLE $Table CHANGE $Tag->{NameNew} $Tag->{NameNew} $Tag->{Type} DEFAULT NULL";
             }
 
             # investigate the default value
@@ -499,7 +555,7 @@ sub IndexCreate {
     # check needed stuff
     for (qw(TableName Name Data)) {
         if ( !$Param{$_} ) {
-            $Self->{LogObject}->Log(
+            $Kernel::OM->Get('Kernel::System::Log')->Log(
                 Priority => 'error',
                 Message  => "Need $_!"
             );
@@ -507,22 +563,31 @@ sub IndexCreate {
         }
     }
 
-    my $SQL   = "CREATE INDEX $Param{Name} ON $Param{TableName} (";
-    my @Array = @{ $Param{Data} };
+    my $CreateIndexSQL = "CREATE INDEX $Param{Name} ON $Param{TableName} (";
+    my @Array          = @{ $Param{Data} };
     for ( 0 .. $#Array ) {
         if ( $_ > 0 ) {
-            $SQL .= ', ';
+            $CreateIndexSQL .= ', ';
         }
-        $SQL .= $Array[$_]->{Name};
+        $CreateIndexSQL .= $Array[$_]->{Name};
         if ( $Array[$_]->{Size} ) {
-            $SQL .= "($Array[$_]->{Size})";
+            $CreateIndexSQL .= "($Array[$_]->{Size})";
         }
     }
-    $SQL .= ')';
+    $CreateIndexSQL .= ')';
+
+    my @SQL;
+
+    # create index only if it does not exist already
+    push @SQL,
+        "SET \@IndexExists := (SELECT COUNT(*) FROM information_schema.statistics WHERE table_schema = DATABASE() AND table_name = '$Param{TableName}' AND index_name = '$Param{Name}')";
+    push @SQL,
+        "SET \@IndexSQLStatement := IF( \@IndexExists = 0, '$CreateIndexSQL', 'SELECT ''INFO: Index $Param{Name} already exists, skipping.''' )";
+    push @SQL, "PREPARE IndexStatement FROM \@IndexSQLStatement";
+    push @SQL, "EXECUTE IndexStatement";
 
     # return SQL
-    return ($SQL);
-
+    return @SQL;
 }
 
 sub IndexDrop {
@@ -531,15 +596,28 @@ sub IndexDrop {
     # check needed stuff
     for (qw(TableName Name)) {
         if ( !$Param{$_} ) {
-            $Self->{LogObject}->Log(
+            $Kernel::OM->Get('Kernel::System::Log')->Log(
                 Priority => 'error',
-                Message  => "Need $_!"
+                Message  => "Need $_!",
             );
             return;
         }
     }
-    my $SQL = 'DROP INDEX ' . $Param{Name} . ' ON ' . $Param{TableName};
-    return ($SQL);
+
+    my $DropIndexSQL = 'DROP INDEX ' . $Param{Name} . ' ON ' . $Param{TableName};
+
+    my @SQL;
+
+    # drop index only if it does still exist
+    push @SQL,
+        "SET \@IndexExists := (SELECT COUNT(*) FROM information_schema.statistics WHERE table_schema = DATABASE() AND table_name = '$Param{TableName}' AND index_name = '$Param{Name}')";
+    push @SQL,
+        "SET \@IndexSQLStatement := IF( \@IndexExists > 0, '$DropIndexSQL', 'SELECT ''INFO: Index $Param{Name} does not exist, skipping.''' )";
+    push @SQL, "PREPARE IndexStatement FROM \@IndexSQLStatement";
+    push @SQL, "EXECUTE IndexStatement";
+
+    # return SQL
+    return @SQL;
 }
 
 sub ForeignKeyCreate {
@@ -548,7 +626,7 @@ sub ForeignKeyCreate {
     # check needed stuff
     for (qw(LocalTableName Local ForeignTableName Foreign)) {
         if ( !$Param{$_} ) {
-            $Self->{LogObject}->Log(
+            $Kernel::OM->Get('Kernel::System::Log')->Log(
                 Priority => 'error',
                 Message  => "Need $_!"
             );
@@ -559,19 +637,29 @@ sub ForeignKeyCreate {
     # create foreign key name
     my $ForeignKey = "FK_$Param{LocalTableName}_$Param{Local}_$Param{Foreign}";
     if ( length($ForeignKey) > 60 ) {
-        my $MD5 = $Self->{MainObject}->MD5sum(
+        my $MD5 = $Kernel::OM->Get('Kernel::System::Main')->MD5sum(
             String => $ForeignKey,
         );
         $ForeignKey = substr $ForeignKey, 0, 58;
         $ForeignKey .= substr $MD5, 0,  1;
-        $ForeignKey .= substr $MD5, 61, 1;
+        $ForeignKey .= substr $MD5, 31, 1;
     }
 
     # add foreign key
-    my $SQL = "ALTER TABLE $Param{LocalTableName} ADD CONSTRAINT $ForeignKey FOREIGN KEY "
+    my $CreateForeignKeySQL = "ALTER TABLE $Param{LocalTableName} ADD CONSTRAINT $ForeignKey FOREIGN KEY "
         . "($Param{Local}) REFERENCES $Param{ForeignTableName} ($Param{Foreign})";
 
-    return ($SQL);
+    my @SQL;
+
+    # create foreign key
+    push @SQL,
+        "SET \@FKExists := (SELECT COUNT(*) FROM information_schema.table_constraints WHERE table_schema = DATABASE() AND table_name = '$Param{LocalTableName}' AND constraint_name = '$ForeignKey')";
+    push @SQL,
+        "SET \@FKSQLStatement := IF( \@FKExists = 0, '$CreateForeignKeySQL', 'SELECT ''INFO: Foreign key constraint $ForeignKey does already exist, skipping.''' )";
+    push @SQL, "PREPARE FKStatement FROM \@FKSQLStatement";
+    push @SQL, "EXECUTE FKStatement";
+
+    return @SQL;
 }
 
 sub ForeignKeyDrop {
@@ -580,7 +668,7 @@ sub ForeignKeyDrop {
     # check needed stuff
     for (qw(LocalTableName Local ForeignTableName Foreign)) {
         if ( !$Param{$_} ) {
-            $Self->{LogObject}->Log(
+            $Kernel::OM->Get('Kernel::System::Log')->Log(
                 Priority => 'error',
                 Message  => "Need $_!"
             );
@@ -591,24 +679,23 @@ sub ForeignKeyDrop {
     # create foreign key name
     my $ForeignKey = "FK_$Param{LocalTableName}_$Param{Local}_$Param{Foreign}";
     if ( length($ForeignKey) > 60 ) {
-        my $MD5 = $Self->{MainObject}->MD5sum(
+        my $MD5 = $Kernel::OM->Get('Kernel::System::Main')->MD5sum(
             String => $ForeignKey,
         );
         $ForeignKey = substr $ForeignKey, 0, 58;
         $ForeignKey .= substr $MD5, 0,  1;
-        $ForeignKey .= substr $MD5, 61, 1;
+        $ForeignKey .= substr $MD5, 31, 1;
     }
 
-    # drop foreign key
     my @SQL;
-
-    if ( $Self->{ConfigObject}->Get('Database::ShellOutput') ) {
+    if ( $Kernel::OM->Get('Kernel::Config')->Get('Database::ShellOutput') ) {
         push @SQL, $Self->{'DB::Comment'}
             . ' MySQL does not create foreign key constraints in MyISAM. Dropping nonexisting constraints in MyISAM works just fine.';
         push @SQL, $Self->{'DB::Comment'}
             . ' However, if the table is converted to InnoDB, this will result in an error. Therefore, only drop constraints if they exist.';
     }
 
+    # drop foreign key
     push @SQL,
         "SET \@FKExists := (SELECT COUNT(*) FROM information_schema.table_constraints WHERE table_schema = DATABASE() AND table_name = '$Param{LocalTableName}' AND constraint_name = '$ForeignKey')";
     push @SQL,
@@ -616,6 +703,14 @@ sub ForeignKeyDrop {
     push @SQL, "PREPARE FKStatement FROM \@FKSQLStatement";
     push @SQL, "EXECUTE FKStatement";
 
+    # drop index with the same name like the foreign key
+    push @SQL,
+        "SET \@IndexExists := (SELECT COUNT(*) FROM information_schema.statistics WHERE table_schema = DATABASE() AND table_name = '$Param{LocalTableName}' AND index_name = '$ForeignKey')";
+    push @SQL,
+        "SET \@IndexSQLStatement := IF( \@IndexExists > 0, 'DROP INDEX $ForeignKey ON $Param{LocalTableName}', 'SELECT ''INFO: Index $ForeignKey does not exist, skipping.''' )";
+    push @SQL, "PREPARE IndexStatement FROM \@IndexSQLStatement";
+    push @SQL, "EXECUTE IndexStatement";
+
     return @SQL;
 }
 
@@ -625,26 +720,34 @@ sub UniqueCreate {
     # check needed stuff
     for (qw(TableName Name Data)) {
         if ( !$Param{$_} ) {
-            $Self->{LogObject}->Log(
+            $Kernel::OM->Get('Kernel::System::Log')->Log(
                 Priority => 'error',
                 Message  => "Need $_!"
             );
             return;
         }
     }
-    my $SQL   = "ALTER TABLE $Param{TableName} ADD CONSTRAINT $Param{Name} UNIQUE INDEX (";
-    my @Array = @{ $Param{Data} };
+    my $CreateUniqueSQL = "ALTER TABLE $Param{TableName} ADD CONSTRAINT $Param{Name} UNIQUE INDEX (";
+    my @Array           = @{ $Param{Data} };
     for ( 0 .. $#Array ) {
         if ( $_ > 0 ) {
-            $SQL .= ', ';
+            $CreateUniqueSQL .= ', ';
         }
-        $SQL .= $Array[$_]->{Name};
+        $CreateUniqueSQL .= $Array[$_]->{Name};
     }
-    $SQL .= ')';
+    $CreateUniqueSQL .= ')';
 
-    # return SQL
-    return ($SQL);
+    my @SQL;
 
+    # create unique constraint
+    push @SQL,
+        "SET \@UniqueExists := (SELECT COUNT(*) FROM information_schema.table_constraints WHERE table_schema = DATABASE() AND table_name = '$Param{TableName}' AND constraint_name = '$Param{Name}')";
+    push @SQL,
+        "SET \@UniqueSQLStatement := IF( \@UniqueExists = 0, '$CreateUniqueSQL', 'SELECT ''INFO: Unique constraint $Param{Name} does already exist, skipping.''' )";
+    push @SQL, "PREPARE UniqueStatement FROM \@UniqueSQLStatement";
+    push @SQL, "EXECUTE UniqueStatement";
+
+    return @SQL;
 }
 
 sub UniqueDrop {
@@ -653,26 +756,41 @@ sub UniqueDrop {
     # check needed stuff
     for (qw(TableName Name)) {
         if ( !$Param{$_} ) {
-            $Self->{LogObject}->Log(
+            $Kernel::OM->Get('Kernel::System::Log')->Log(
                 Priority => 'error',
                 Message  => "Need $_!"
             );
             return;
         }
     }
-    my $SQL = 'ALTER TABLE ' . $Param{TableName} . ' DROP INDEX ' . $Param{Name};
-    return ($SQL);
+    my $DropUniqueSQL = 'ALTER TABLE ' . $Param{TableName} . ' DROP INDEX ' . $Param{Name};
+
+    my @SQL;
+
+    # create unique constraint
+    push @SQL,
+        "SET \@UniqueExists := (SELECT COUNT(*) FROM information_schema.table_constraints WHERE table_schema = DATABASE() AND table_name = '$Param{TableName}' AND constraint_name = '$Param{Name}')";
+    push @SQL,
+        "SET \@UniqueSQLStatement := IF( \@UniqueExists > 0, '$DropUniqueSQL', 'SELECT ''INFO: Unique constraint $Param{Name} does not exist, skipping.''' )";
+    push @SQL, "PREPARE UniqueStatement FROM \@UniqueSQLStatement";
+    push @SQL, "EXECUTE UniqueStatement";
+
+    return @SQL;
 }
 
 sub Insert {
     my ( $Self, @Param ) = @_;
 
+    # get needed objects
+    my $ConfigObject   = $Kernel::OM->Get('Kernel::Config');
+    my $DateTimeObject = $Kernel::OM->Create('Kernel::System::DateTime');
+
     my $SQL    = '';
     my @Keys   = ();
     my @Values = ();
     for my $Tag (@Param) {
         if ( $Tag->{Tag} eq 'Insert' && $Tag->{TagType} eq 'Start' ) {
-            if ( $Self->{ConfigObject}->Get('Database::ShellOutput') ) {
+            if ( $ConfigObject->Get('Database::ShellOutput') ) {
                 $SQL .= $Self->{'DB::Comment'}
                     . "----------------------------------------------------------\n";
                 $SQL .= $Self->{'DB::Comment'} . " insert into table $Tag->{Table}\n";
@@ -687,7 +805,7 @@ sub Insert {
             my $Value;
             if ( defined $Tag->{Value} ) {
                 $Value = $Tag->{Value};
-                $Self->{LogObject}->Log(
+                $Kernel::OM->Get('Kernel::System::Log')->Log(
                     Priority => 'error',
                     Message  => 'The content for inserts is not longer appreciated '
                         . 'attribut Value, use Content from now on! Reason: You can\'t '
@@ -722,18 +840,15 @@ sub Insert {
             $Value .= ', ';
         }
         if ( $Tmp eq 'current_timestamp' ) {
-            if ( $Self->{ConfigObject}->Get('Database::ShellOutput') ) {
+            if ( $ConfigObject->Get('Database::ShellOutput') ) {
                 $Value .= $Tmp;
             }
             else {
-                my $Timestamp = $Self->{TimeObject}->CurrentTimestamp();
+                my $Timestamp = $DateTimeObject->ToString();
                 $Value .= '\'' . $Timestamp . '\'';
             }
         }
         else {
-            if ( $Self->{ConfigObject}->Get('Database::ShellOutput') ) {
-                $Tmp =~ s/\n/\r/g;
-            }
             $Value .= $Tmp;
         }
     }
