@@ -1,9 +1,9 @@
 # --
-# Copyright (C) 2001-2019 OTRS AG, https://otrs.com/
+# Copyright (C) 2001-2017 OTRS AG, http://otrs.com/
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
-# the enclosed file COPYING for license information (GPL). If you
-# did not receive this file, see https://www.gnu.org/licenses/gpl-3.0.txt.
+# the enclosed file COPYING for license information (AGPL). If you
+# did not receive this file, see http://www.gnu.org/licenses/agpl.txt.
 # --
 
 package Kernel::System::Daemon::DaemonModules::BaseTaskWorker;
@@ -21,17 +21,15 @@ our @ObjectDependencies = (
 
 Kernel::System::Daemon::DaemonModules::BaseTaskWorker - scheduler task worker base class
 
-=head1 SYNOPSIS
+=head1 DESCRIPTION
 
 Base class for scheduler daemon task worker modules.
 
 =head1 PUBLIC INTERFACE
 
-=over 4
+=begin Internal:
 
-=cut
-
-=item _HandleError()
+=head2 _HandleError()
 
 Creates a system error message and sends an email with the error messages form a task execution.
 
@@ -71,13 +69,14 @@ sub _HandleError {
             Body     => $Param{ErrorMessage},
         );
 
-        return $Sent;
+        return 1 if $Sent->{Success};
+        return;
     }
 
     return;
 }
 
-=item _CheckTaskParams()
+=head2 _CheckTaskParams()
 
 Performs basic checks for common task parameters.
 
@@ -147,14 +146,14 @@ sub _CheckTaskParams {
 }
 1;
 
-=back
+=end Internal:
 
 =head1 TERMS AND CONDITIONS
 
-This software is part of the OTRS project (L<https://otrs.org/>).
+This software is part of the OTRS project (L<http://otrs.org/>).
 
 This software comes with ABSOLUTELY NO WARRANTY. For details, see
-the enclosed file COPYING for license information (GPL). If you
-did not receive this file, see L<https://www.gnu.org/licenses/gpl-3.0.txt>.
+the enclosed file COPYING for license information (AGPL). If you
+did not receive this file, see L<http://www.gnu.org/licenses/agpl.txt>.
 
 =cut
