@@ -14,34 +14,29 @@ use utf8;
 
 use Kernel::System::VariableCheck qw(:all);
 
-use base qw(Kernel::System::ProcessManagement::TransitionAction::Base);
+use parent qw(Kernel::System::ProcessManagement::TransitionAction::Base);
 
 our @ObjectDependencies = (
     'Kernel::System::Log',
     'Kernel::System::Service',
     'Kernel::System::Ticket',
+    'Kernel::Config',
 );
 
 =head1 NAME
 
 Kernel::System::ProcessManagement::TransitionAction::TicketServiceSet - A module to set the ticket Service
 
-=head1 SYNOPSIS
+=head1 DESCRIPTION
 
 All TicketServiceSet functions.
 
 =head1 PUBLIC INTERFACE
 
-=over 4
+=head2 new()
 
-=cut
+Don't use the constructor directly, use the ObjectManager instead:
 
-=item new()
-
-create an object. Do not use it directly, instead use:
-
-    use Kernel::System::ObjectManager;
-    local $Kernel::OM = Kernel::System::ObjectManager->new();
     my $TicketServiceSetObject = $Kernel::OM->Get('Kernel::System::ProcessManagement::TransitionAction::TicketServiceSet');
 
 =cut
@@ -56,7 +51,7 @@ sub new {
     return $Self;
 }
 
-=item Run()
+=head2 Run()
 
     Run Data
 
@@ -111,7 +106,11 @@ sub Run {
         return;
     }
 
-    if ( !$Param{Ticket}->{CustomerUserID} ) {
+    if (
+        !$Param{Ticket}->{CustomerUserID}
+        && !$Kernel::OM->Get('Kernel::Config')->Get('Ticket::Service::Default::UnknownCustomer')
+        )
+    {
         $Kernel::OM->Get('Kernel::System::Log')->Log(
             Priority => 'error',
             Message  => $CommonMessage . "To set a service the ticket requires a customer!",
@@ -268,7 +267,9 @@ sub Run {
     return $Success;
 }
 
-=item _CheckService()
+=begin Internal:
+
+=head2 _CheckService()
 
 checks if a service is assigned to a customer user
 
@@ -280,6 +281,7 @@ checks if a service is assigned to a customer user
     Returns:
 
     $Success = 1;       # or undef
+
 =cut
 
 sub _CheckService {
@@ -304,7 +306,7 @@ sub _CheckService {
 
 1;
 
-=back
+=end Internal:
 
 =head1 TERMS AND CONDITIONS
 
