@@ -1,9 +1,9 @@
 # --
-# Copyright (C) 2001-2019 OTRS AG, https://otrs.com/
+# Copyright (C) 2001-2017 OTRS AG, http://otrs.com/
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
-# the enclosed file COPYING for license information (GPL). If you
-# did not receive this file, see https://www.gnu.org/licenses/gpl-3.0.txt.
+# the enclosed file COPYING for license information (AGPL). If you
+# did not receive this file, see http://www.gnu.org/licenses/agpl.txt.
 # --
 
 package Kernel::System::Scheduler;
@@ -20,22 +20,16 @@ our @ObjectDependencies = (
 
 Kernel::System::Scheduler - Scheduler lib
 
-=head1 SYNOPSIS
+=head1 DESCRIPTION
 
 Includes the functions to add a new task to the scheduler daemon.
 
 =head1 PUBLIC INTERFACE
 
-=over 4
-
-=cut
-
-=item new()
+=head2 new()
 
 create a scheduler object. Do not use it directly, instead use:
 
-    use Kernel::System::ObjectManager;
-    local $Kernel::OM = Kernel::System::ObjectManager->new();
     my $SchedulerObject = $Kernel::OM->Get('Kernel::System::Scheduler');
 
 =cut
@@ -50,7 +44,7 @@ sub new {
     return $Self;
 }
 
-=item TaskAdd()
+=head2 TaskAdd()
 
 add a task to scheduler
 
@@ -100,7 +94,7 @@ sub TaskAdd {
     return;
 }
 
-=item FutureTaskList()
+=head2 FutureTaskList()
 
 get the list of scheduler future tasks
 
@@ -136,7 +130,39 @@ sub FutureTaskList {
     return @List;
 }
 
-=item FutureTaskDelete()
+=head2 TaskList()
+
+get the list of scheduler tasks
+
+    my @List = $SchedulerObject->TaskList(
+        Type => 'some type',  # optional
+    );
+
+Returns:
+
+    @List = (
+        {
+            TaskID => 123,
+            Name   => 'any name',
+            Type   => 'GenericInterface',
+        },
+        {
+            TaskID => 456,
+            Name   => 'any other name',
+            Type   => 'GenericInterface',
+        },
+        # ...
+    );
+
+=cut
+
+sub TaskList {
+    my ( $Self, %Param ) = @_;
+
+    return $Kernel::OM->Get('Kernel::System::Daemon::SchedulerDB')->TaskList(%Param);
+}
+
+=head2 FutureTaskDelete()
 
 delete a task from scheduler future task list
 
@@ -165,14 +191,12 @@ sub FutureTaskDelete {
 
 1;
 
-=back
-
 =head1 TERMS AND CONDITIONS
 
-This software is part of the OTRS project (L<https://otrs.org/>).
+This software is part of the OTRS project (L<http://otrs.org/>).
 
 This software comes with ABSOLUTELY NO WARRANTY. For details, see
-the enclosed file COPYING for license information (GPL). If you
-did not receive this file, see L<https://www.gnu.org/licenses/gpl-3.0.txt>.
+the enclosed file COPYING for license information (AGPL). If you
+did not receive this file, see L<http://www.gnu.org/licenses/agpl.txt>.
 
 =cut
