@@ -1,9 +1,9 @@
 # --
-# Copyright (C) 2001-2019 OTRS AG, https://otrs.com/
+# Copyright (C) 2001-2017 OTRS AG, http://otrs.com/
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
-# the enclosed file COPYING for license information (GPL). If you
-# did not receive this file, see https://www.gnu.org/licenses/gpl-3.0.txt.
+# the enclosed file COPYING for license information (AGPL). If you
+# did not receive this file, see http://www.gnu.org/licenses/agpl.txt.
 # --
 
 package Kernel::System::Ticket::Event::NotificationEvent::Transport::Email;
@@ -16,46 +16,40 @@ use warnings;
 use Kernel::System::VariableCheck qw(:all);
 use Kernel::Language qw(Translatable);
 
-use base qw(Kernel::System::Ticket::Event::NotificationEvent::Transport::Base);
+use parent qw(Kernel::System::Ticket::Event::NotificationEvent::Transport::Base);
 
 our @ObjectDependencies = (
     'Kernel::Config',
     'Kernel::Output::HTML::Layout',
     'Kernel::System::CustomerUser',
+    'Kernel::System::Crypt::PGP',
+    'Kernel::System::Crypt::SMIME',
     'Kernel::System::Email',
     'Kernel::System::Log',
     'Kernel::System::Main',
     'Kernel::System::Queue',
     'Kernel::System::SystemAddress',
     'Kernel::System::Ticket',
+    'Kernel::System::Ticket::Article',
+    'Kernel::System::Ticket::Article::Backend::Email',
     'Kernel::System::User',
     'Kernel::System::Web::Request',
-    'Kernel::System::Crypt::PGP',
-    'Kernel::System::Crypt::SMIME',
-    'Kernel::System::DynamicField',
-    'Kernel::System::DynamicField::Backend',
 );
 
 =head1 NAME
 
 Kernel::System::Ticket::Event::NotificationEvent::Transport::Email - email transport layer
 
-=head1 SYNOPSIS
+=head1 DESCRIPTION
 
 Notification event transport layer.
 
 =head1 PUBLIC INTERFACE
 
-=over 4
-
-=cut
-
-=item new()
+=head2 new()
 
 create a notification transport object. Do not use it directly, instead use:
 
-    use Kernel::System::ObjectManager;
-    local $Kernel::OM = Kernel::System::ObjectManager->new('');
     my $TransportObject = $Kernel::OM->Get('Kernel::System::Ticket::Event::NotificationEvent::Transport::Email');
 
 =cut
@@ -146,18 +140,6 @@ sub SendNotification {
             $EmailTemplate = 'Default';
         }
 
-        my $RecipientLanugageObject = $LayoutObject->{LanguageObject};
-
-        if ( $Recipient{UserLanguage} ) {
-
-            # Create new language object to include recipient language preference if applicable.
-            $RecipientLanugageObject = Kernel::Language->new(
-                UserLanguage => $Recipient{UserLanguage},
-            );
-        }
-
-        local $LayoutObject->{LanguageObject} = $RecipientLanugageObject;
-
         # generate HTML
         $Notification{Body} = $LayoutObject->Output(
             TemplateFile => "NotificationEvent/Email/$EmailTemplate",
@@ -186,19 +168,19 @@ sub SendNotification {
         my $EmailObject = $Kernel::OM->Get('Kernel::System::Email');
 
         my $Sent = $EmailObject->Send(
-            From       => $From,
-            To         => $Recipient{UserEmail},
-            Subject    => $Notification{Subject},
-            MimeType   => $Notification{ContentType},
-            Type       => $Notification{ContentType},
-            Charset    => 'utf-8',
-            Body       => $Notification{Body},
-            Loop       => 1,
-            Attachment => $Param{Attachments},
-            %{$SecurityOptions},
+            From          => $From,
+            To            => $Recipient{UserEmail},
+            Subject       => $Notification{Subject},
+            MimeType      => $Notification{ContentType},
+            Type          => $Notification{ContentType},
+            Charset       => 'utf-8',
+            Body          => $Notification{Body},
+            Loop          => 1,
+            Attachment    => $Param{Attachments},
+            EmailSecurity => $SecurityOptions || {},
         );
 
-        if ( !$Sent ) {
+        if ( !$Sent->{Success} ) {
             $Kernel::OM->Get('Kernel::System::Log')->Log(
                 Priority => 'error',
                 Message  => "'$Notification{Name}' notification could not be sent to agent '$Recipient{UserEmail} ",
@@ -224,35 +206,66 @@ sub SendNotification {
     }
     else {
 
-        # get queue object
-        my $QueueObject = $Kernel::OM->Get('Kernel::System::Queue');
+        my $QueueObject   = $Kernel::OM->Get('Kernel::System::Queue');
+        my $ArticleObject = $Kernel::OM->Get('Kernel::System::Ticket::Article');
 
-        my $QueueID;
+        # Get last article from customer.
+        my @CustomerArticles = $ArticleObject->ArticleList(
+            TicketID   => $Param{TicketID},
+            SenderType => 'customer',
+            OnlyLast   => 1,
+        );
 
-        # get article
-        my %Article = $TicketObject->ArticleLastCustomerArticle(
+        my %CustomerArticle;
+
+        ARTICLE:
+        for my $Article (@CustomerArticles) {
+            next ARTICLE if !$Article->{ArticleID};
+
+            %CustomerArticle = $ArticleObject->BackendForArticle( %{$Article} )->ArticleGet(
+                %{$Article},
+                DynamicFields => 0,
+            );
+        }
+
+        my %Article = %CustomerArticle;
+
+        # If the ticket has no customer article, get the last agent article.
+        if ( !%CustomerArticle ) {
+
+            # Get last article from agent.
+            my @AgentArticles = $ArticleObject->ArticleList(
+                TicketID   => $Param{TicketID},
+                SenderType => 'agent',
+                OnlyLast   => 1,
+            );
+
+            my %AgentArticle;
+
+            ARTICLE:
+            for my $Article (@AgentArticles) {
+                next ARTICLE if !$Article->{ArticleID};
+
+                %AgentArticle = $ArticleObject->BackendForArticle( %{$Article} )->ArticleGet(
+                    %{$Article},
+                    DynamicFields => 0,
+                );
+            }
+
+            %Article = %AgentArticle;
+        }
+
+        # Get raw ticket data.
+        my %Ticket = $TicketObject->TicketGet(
             TicketID      => $Param{TicketID},
             DynamicFields => 0,
         );
 
-        # set "From" address from Article if exist, otherwise use ticket information, see bug# 9035
-        if (%Article) {
-            $QueueID = $Article{QueueID};
-        }
-        else {
-
-            # get ticket data
-            my %Ticket = $TicketObject->TicketGet(
-                TicketID => $Param{TicketID},
-            );
-            $QueueID = $Ticket{QueueID};
-        }
-
-        my %Address = $QueueObject->GetSystemAddress( QueueID => $QueueID );
+        my %Address = $QueueObject->GetSystemAddress( QueueID => $Ticket{QueueID} );
 
         # get queue
         my %Queue = $QueueObject->QueueGet(
-            ID => $QueueID,
+            ID => $Ticket{QueueID},
         );
 
         # security part
@@ -263,33 +276,28 @@ sub SendNotification {
         );
         return if !$SecurityOptions;
 
-        my $ArticleType = 'email-notification-ext';
-
-        if ( IsArrayRefWithData( $Param{Notification}->{Data}->{NotificationArticleTypeID} ) ) {
-
-            # get notification article type
-            $ArticleType = $TicketObject->ArticleTypeLookup(
-                ArticleTypeID => $Param{Notification}->{Data}->{NotificationArticleTypeID}->[0],
-            );
+        my $IsVisibleForCustomer = 1;
+        if ( defined $Notification{Data}->{IsVisibleForCustomer} ) {
+            $IsVisibleForCustomer = $Notification{Data}->{IsVisibleForCustomer}->[0];
         }
 
-        my $ArticleID = $TicketObject->ArticleSend(
-            ArticleType    => $ArticleType,
-            SenderType     => 'system',
-            TicketID       => $Param{TicketID},
-            HistoryType    => 'SendCustomerNotification',
-            HistoryComment => "\%\%$Recipient{UserEmail}",
-            From           => "$Address{RealName} <$Address{Email}>",
-            To             => $Recipient{UserEmail},
-            Subject        => $Notification{Subject},
-            Body           => $Notification{Body},
-            MimeType       => $Notification{ContentType},
-            Type           => $Notification{ContentType},
-            Charset        => 'utf-8',
-            UserID         => $Param{UserID},
-            Loop           => 1,
-            Attachment     => $Param{Attachments},
-            %{$SecurityOptions},
+        my $ArticleID = $ArticleObject->BackendForChannel( ChannelName => 'Email' )->ArticleSend(
+            TicketID             => $Param{TicketID},
+            SenderType           => 'system',
+            IsVisibleForCustomer => $IsVisibleForCustomer,
+            HistoryType          => 'SendCustomerNotification',
+            HistoryComment       => "\%\%$Recipient{UserEmail}",
+            From                 => "$Address{RealName} <$Address{Email}>",
+            To                   => $Recipient{UserEmail},
+            Subject              => $Notification{Subject},
+            Body                 => $Notification{Body},
+            MimeType             => $Notification{ContentType},
+            Type                 => $Notification{ContentType},
+            Charset              => 'utf-8',
+            UserID               => $Param{UserID},
+            Loop                 => 1,
+            Attachment           => $Param{Attachments},
+            EmailSecurity        => $SecurityOptions || {},
         );
 
         if ( !$ArticleID ) {
@@ -352,10 +360,8 @@ sub GetTransportRecipients {
             $Recipient{UserEmail} = $RecipientEmail;
 
             # check if we have a specified article type
-            if ( $Param{Notification}->{Data}->{NotificationArticleTypeID} ) {
-                $Recipient{NotificationArticleType} = $Kernel::OM->Get('Kernel::System::Ticket')->ArticleTypeLookup(
-                    ArticleTypeID => $Param{Notification}->{Data}->{NotificationArticleTypeID}->[0]
-                ) || 'email-notification-ext';
+            if ( $Param{Notification}->{Data}->{IsVisibleForCustomer} ) {
+                $Recipient{IsVisibleForCustomer} = 1;
             }
 
             # check recipients
@@ -404,27 +410,11 @@ sub TransportSettingsDisplayGet {
     # get layout object
     my $LayoutObject = $Kernel::OM->Get('Kernel::Output::HTML::Layout');
 
-    # Display article types for article creation if notification is sent
-    # only use 'email-notification-*'-type articles
-    my %NotificationArticleTypes = $Kernel::OM->Get('Kernel::System::Ticket')->ArticleTypeList( Result => 'HASH' );
-    for my $NotifArticleTypeID ( sort keys %NotificationArticleTypes ) {
-        if ( $NotificationArticleTypes{$NotifArticleTypeID} !~ /^email-notification-/ ) {
-            delete $NotificationArticleTypes{$NotifArticleTypeID};
-        }
-    }
-    $Param{NotificationArticleTypesStrg} = $LayoutObject->BuildSelection(
-        Data        => \%NotificationArticleTypes,
-        Name        => 'NotificationArticleTypeID',
-        Translation => 1,
-        SelectedID  => $Param{Data}->{NotificationArticleTypeID},
-        Class       => 'Modernize W50pc',
-    );
-
     $Param{TransportEmailTemplateStrg} = $LayoutObject->BuildSelection(
         Data        => \%Templates,
         Name        => 'TransportEmailTemplate',
         Translation => 0,
-        SelectedID  => $Param{Data}->{TransportEmailTemplate},
+        SelectedID  => $Param{Data}->{TransportEmailTemplate} || 'Default',
         Class       => 'Modernize W50pc',
     );
 
@@ -449,7 +439,7 @@ sub TransportSettingsDisplayGet {
 
     # set security settings enabled
     $Param{EmailSecuritySettings} = ( $Param{Data}->{EmailSecuritySettings} ? 'checked="checked"' : '' );
-    $Param{SecurityDisabled}      = 0;
+    $Param{SecurityDisabled} = 0;
 
     if ( $Param{EmailSecuritySettings} eq '' ) {
         $Param{SecurityDisabled} = 1;
@@ -585,16 +575,28 @@ sub SecurityOptionsGet {
     my $NotificationSenderEmail = $Param{FromEmail};
 
     # Define security options container
-    my %SecurityOptions;
+    my %SecurityOptions = (
+        Method => 'Detached',
+    );
+
     my @SignKeys;
     my @EncryptKeys;
     my $KeyField;
-    my $Method;
-    my $Subtype = 'Detached';
 
     # Get private and public keys for the given backend (PGP or SMIME)
     if ( $SignEncryptNotification =~ /^PGP/i ) {
-        @SignKeys = $Kernel::OM->Get('Kernel::System::Crypt::PGP')->PrivateKeySearch(
+
+        my $PGPObject = $Kernel::OM->Get('Kernel::System::Crypt::PGP');
+
+        if ( !$PGPObject ) {
+            $Kernel::OM->Get('Kernel::System::Log')->Log(
+                Priority => 'error',
+                Message  => "No PGP support!",
+            );
+            return;
+        }
+
+        @SignKeys = $PGPObject->PrivateKeySearch(
             Search => $NotificationSenderEmail,
         );
 
@@ -602,7 +604,7 @@ sub SecurityOptionsGet {
         @SignKeys = grep { $_->{Status} eq 'good' } @SignKeys;
 
         # get public keys
-        @EncryptKeys = $Kernel::OM->Get('Kernel::System::Crypt::PGP')->PublicKeySearch(
+        @EncryptKeys = $PGPObject->PublicKeySearch(
             Search => $Param{Recipient}->{UserEmail},
         );
 
@@ -611,12 +613,24 @@ sub SecurityOptionsGet {
 
         # Get PGP method (Detached or In-line).
         if ( !$Kernel::OM->Get('Kernel::Output::HTML::Layout')->{BrowserRichText} ) {
-            $Subtype = $Kernel::OM->Get('Kernel::Config')->Get('PGP::Method') || 'Detached';
+            $SecurityOptions{Method} = $Kernel::OM->Get('Kernel::Config')->Get('PGP::Method') || 'Detached';
         }
-        $Method   = 'PGP';
+
+        $SecurityOptions{Backend} = 'PGP';
         $KeyField = 'Key';
     }
     elsif ( $SignEncryptNotification =~ /^SMIME/i ) {
+
+        my $SMIMEObject = $Kernel::OM->Get('Kernel::System::Crypt::SMIME');
+
+        if ( !$SMIMEObject ) {
+            $Kernel::OM->Get('Kernel::System::Log')->Log(
+                Priority => 'error',
+                Message  => "No SMIME support!",
+            );
+            return;
+        }
+
         @SignKeys = $Kernel::OM->Get('Kernel::System::Crypt::SMIME')->PrivateSearch(
             Search => $NotificationSenderEmail,
         );
@@ -625,7 +639,7 @@ sub SecurityOptionsGet {
             Search => $Param{Recipient}->{UserEmail},
         );
 
-        $Method   = 'SMIME';
+        $SecurityOptions{Backend} = 'SMIME';
         $KeyField = 'Filename';
     }
 
@@ -680,7 +694,7 @@ sub SecurityOptionsGet {
         if ( !IsHashRefWithData( \%SignKey ) ) {
 
             my $Message
-                = "Could not sign notification '$Param{Notification}->{Name}' due to missing $Method sign key for '$NotificationSenderEmail'";
+                = "Could not sign notification '$Param{Notification}->{Name}' due to missing $SecurityOptions{Backend} sign key for '$NotificationSenderEmail'";
 
             if ( $OnMissingSigningKeys eq 'Skip' ) {
 
@@ -702,11 +716,7 @@ sub SecurityOptionsGet {
 
         # Add signature option if a sign key is available
         else {
-            $SecurityOptions{Sign} = {
-                Type    => $Method,
-                SubType => $Subtype,
-                Key     => $SignKey{$KeyField},
-            };
+            $SecurityOptions{SignKey} = $SignKey{$KeyField};
         }
     }
 
@@ -719,7 +729,7 @@ sub SecurityOptionsGet {
         if ( !IsHashRefWithData( \%EncryptKey ) ) {
 
             my $Message
-                = "Could not encrypt notification '$Param{Notification}->{Name}' due to missing $Method encryption key for '$Param{Recipient}->{UserEmail}'";
+                = "Could not encrypt notification '$Param{Notification}->{Name}' due to missing $SecurityOptions{Backend} encryption key for '$Param{Recipient}->{UserEmail}'";
 
             if ( $OnMissingEncryptionKeys eq 'Skip' ) {
 
@@ -741,11 +751,7 @@ sub SecurityOptionsGet {
 
         # Add encrypt option if a encrypt key is available
         else {
-            $SecurityOptions{Crypt} = {
-                Type    => $Method,
-                SubType => $Subtype,
-                Key     => $EncryptKey{$KeyField},
-            };
+            $SecurityOptions{EncryptKeys} = [ $EncryptKey{$KeyField}, ];
         }
     }
 
@@ -753,77 +759,14 @@ sub SecurityOptionsGet {
 
 }
 
-sub _ReplaceTicketAttributes {
-    my ( $Self, %Param ) = @_;
-
-    return if !$Param{Field};
-
-    # get needed objects
-    my $DynamicFieldObject        = $Kernel::OM->Get('Kernel::System::DynamicField');
-    my $DynamicFieldBackendObject = $Kernel::OM->Get('Kernel::System::DynamicField::Backend');
-
-    # replace ticket attributes such as <OTRS_Ticket_DynamicField_Name1> or
-    # <OTRS_TICKET_DynamicField_Name1>
-    # <OTRS_Ticket_*> is deprecated and should be removed in further versions of OTRS
-    my $Count = 0;
-    REPLACEMENT:
-    while (
-        $Param{Field}
-        && $Param{Field} =~ m{<OTRS_TICKET_([A-Za-z0-9_]+)>}msxi
-        && $Count++ < 1000
-        )
-    {
-        my $TicketAttribute = $1;
-
-        if ( $TicketAttribute =~ m{DynamicField_(\S+?)_Value} ) {
-            my $DynamicFieldName = $1;
-
-            my $DynamicFieldConfig = $DynamicFieldObject->DynamicFieldGet(
-                Name => $DynamicFieldName,
-            );
-            next REPLACEMENT if !$DynamicFieldConfig;
-
-            # get the display value for each dynamic field
-            my $DisplayValue = $DynamicFieldBackendObject->ValueLookup(
-                DynamicFieldConfig => $DynamicFieldConfig,
-                Key                => $Param{Ticket}->{"DynamicField_$DynamicFieldName"},
-            );
-
-            my $DisplayValueStrg = $DynamicFieldBackendObject->ReadableValueRender(
-                DynamicFieldConfig => $DynamicFieldConfig,
-                Value              => $DisplayValue,
-            );
-
-            $Param{Field} =~ s{<OTRS_TICKET_$TicketAttribute>}{$DisplayValueStrg->{Value} // ''}ige;
-
-            next REPLACEMENT;
-        }
-
-        # if ticket value is scalar substitute all instances (as strings)
-        # this will allow replacements for "<OTRS_TICKET_Title> <OTRS_TICKET_Queue"
-        if ( !ref $Param{Ticket}->{$TicketAttribute} ) {
-            $Param{Field} =~ s{<OTRS_TICKET_$TicketAttribute>}{$Param{Ticket}->{$TicketAttribute} // ''}ige;
-        }
-        else {
-            # if the value is an array (e.g. a multiselect dynamic field) set the value directly
-            # this unfortunately will not let a combination of values to be replaced
-            $Param{Field} = $Param{Ticket}->{$TicketAttribute};
-        }
-    }
-
-    return $Param{Field};
-}
-
 1;
 
-=back
-
 =head1 TERMS AND CONDITIONS
 
-This software is part of the OTRS project (L<https://otrs.org/>).
+This software is part of the OTRS project (L<http://otrs.org/>).
 
 This software comes with ABSOLUTELY NO WARRANTY. For details, see
-the enclosed file COPYING for license information (GPL). If you
-did not receive this file, see L<https://www.gnu.org/licenses/gpl-3.0.txt>.
+the enclosed file COPYING for license information (AGPL). If you
+did not receive this file, see L<http://www.gnu.org/licenses/agpl.txt>.
 
 =cut
