@@ -1,9 +1,9 @@
 # --
-# Copyright (C) 2001-2020 OTRS AG, https://otrs.com/
+# Copyright (C) 2001-2017 OTRS AG, http://otrs.com/
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
-# the enclosed file COPYING for license information (GPL). If you
-# did not receive this file, see https://www.gnu.org/licenses/gpl-3.0.txt.
+# the enclosed file COPYING for license information (AGPL). If you
+# did not receive this file, see http://www.gnu.org/licenses/agpl.txt.
 # --
 
 package Kernel::System::Ticket::IndexAccelerator::RuntimeDB;
@@ -18,9 +18,16 @@ our @ObjectDependencies = (
     'Kernel::System::Lock',
     'Kernel::System::Log',
     'Kernel::System::State',
-    'Kernel::System::Time',
+    'Kernel::System::DateTime',
 );
 
+sub new {
+    my ($Type) = @_;
+
+    my $Self = {};
+    return bless( $Self, $Type );
+}
+
 sub TicketAcceleratorUpdate {
     my ( $Self, %Param ) = @_;
 
@@ -182,7 +189,7 @@ sub TicketAcceleratorIndex {
     # prepare the tickets in Queue bar (all data only with my/your Permission)
     return if !$DBObject->Prepare(
         SQL => "
-            SELECT st.queue_id, sq.name, min(st.create_time_unix), st.ticket_lock_id, count(*)
+            SELECT st.queue_id, sq.name, min(st.create_time), st.ticket_lock_id, count(*)
             FROM ticket st, queue sq
             WHERE st.ticket_state_id IN ( ${\(join ', ', @ViewableStateIDs)} )
                 AND st.queue_id = sq.id
@@ -192,8 +199,7 @@ sub TicketAcceleratorIndex {
             ORDER BY sq.name"
     );
 
-    # get time object
-    my $TimeObject = $Kernel::OM->Get('Kernel::System::Time');
+    my $CurrentDateTimeObject = $Kernel::OM->Create('Kernel::System::DateTime');
 
     my %QueuesSeen;
     while ( my @Row = $DBObject->FetchrowArray() ) {
@@ -221,7 +227,15 @@ sub TicketAcceleratorIndex {
 
             $QueueData->{Count} += $Count;
 
-            my $MaxAge = $TimeObject->SystemTime() - $Row[2];
+            my $TicketCreatedDTObj = $Kernel::OM->Create(
+                'Kernel::System::DateTime',
+                ObjectParams => {
+                    String => $Row[2],
+                },
+            );
+
+            my $Delta = $TicketCreatedDTObj->Delta( DateTimeObject => $CurrentDateTimeObject );
+            my $MaxAge = $Delta->{AbsoluteSeconds};
             $QueueData->{MaxAge} = $MaxAge if $MaxAge > $QueueData->{MaxAge};
 
             # get the oldest queue id
