@@ -1,5 +1,5 @@
 # --
-# Copyright (C) 2001-2018 OTRS AG, http://otrs.com/
+# Copyright (C) 2001-2017 OTRS AG, http://otrs.com/
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
 # the enclosed file COPYING for license information (AGPL). If you
@@ -18,9 +18,17 @@ our @ObjectDependencies = (
     'Kernel::System::Lock',
     'Kernel::System::Log',
     'Kernel::System::State',
-    'Kernel::System::Time',
+    'Kernel::System::Ticket',
+    'Kernel::System::DateTime',
 );
 
+sub new {
+    my ($Type) = @_;
+
+    my $Self = {};
+    return bless( $Self, $Type );
+}
+
 sub TicketAcceleratorUpdate {
     my ( $Self, %Param ) = @_;
 
@@ -38,7 +46,7 @@ sub TicketAcceleratorUpdate {
     # check if ticket is shown or not
     my $IndexUpdateNeeded = 0;
     my $IndexSelected     = 0;
-    my %TicketData        = $Self->TicketGet(
+    my %TicketData        = $Kernel::OM->Get('Kernel::System::Ticket')->TicketGet(
         %Param,
         DynamicFields => 0,
     );
@@ -211,7 +219,7 @@ sub TicketAcceleratorAdd {
     }
 
     # get ticket data
-    my %TicketData = $Self->TicketGet(
+    my %TicketData = $Kernel::OM->Get('Kernel::System::Ticket')->TicketGet(
         %Param,
         DynamicFields => 0,
     );
@@ -243,12 +251,12 @@ sub TicketAcceleratorAdd {
     return if !$Kernel::OM->Get('Kernel::System::DB')->Do(
         SQL => '
             INSERT INTO ticket_index
-                (ticket_id, queue_id, queue, group_id, s_lock, s_state, create_time_unix)
+                (ticket_id, queue_id, queue, group_id, s_lock, s_state, create_time)
                 VALUES (?, ?, ?, ?, ?, ?, ?)',
         Bind => [
             \$Param{TicketID},     \$TicketData{QueueID}, \$TicketData{Queue},
             \$TicketData{GroupID}, \$TicketData{Lock},    \$TicketData{State},
-            \$TicketData{CreateTimeUnix},
+            \$TicketData{Created},
         ],
     );
 
@@ -293,7 +301,7 @@ sub TicketLockAcceleratorAdd {
     }
 
     # get ticket data
-    my %TicketData = $Self->TicketGet(
+    my %TicketData = $Kernel::OM->Get('Kernel::System::Ticket')->TicketGet(
         %Param,
         DynamicFields => 0,
     );
@@ -381,12 +389,13 @@ sub TicketAcceleratorIndex {
         $Type = 'ro';
     }
 
-    my @GroupIDs = $Kernel::OM->Get('Kernel::System::Group')->GroupMemberList(
+    my %GroupList = $Kernel::OM->Get('Kernel::System::Group')->PermissionUserGet(
         UserID => $Param{UserID},
         Type   => $Type,
-        Result => 'ID',
     );
 
+    my @GroupIDs = sort keys %GroupList;
+
     # get index
     $Queues{MaxAge} = 0;
 
@@ -438,15 +447,14 @@ sub TicketAcceleratorIndex {
     # prepare the tickets in Queue bar (all data only with my/your Permission)
     return if !$DBObject->Prepare(
         SQL => "
-            SELECT queue_id, queue, min(create_time_unix), s_lock, count(*)
+            SELECT queue_id, queue, min(create_time), s_lock, count(*)
             FROM ticket_index
             WHERE group_id IN ( ${\(join ', ', @GroupIDs)} )
             GROUP BY queue_id, queue, s_lock
             ORDER BY queue",
     );
 
-    # get time object
-    my $TimeObject = $Kernel::OM->Get('Kernel::System::Time');
+    my $CurrentDateTimeObject = $Kernel::OM->Create('Kernel::System::DateTime');
 
     my %QueuesSeen;
     while ( my @Row = $DBObject->FetchrowArray() ) {
@@ -474,7 +482,15 @@ sub TicketAcceleratorIndex {
 
             $QueueData->{Count} += $Count;
 
-            my $MaxAge = $TimeObject->SystemTime() - $Row[2];
+            my $TicketCreatedDTObj = $Kernel::OM->Create(
+                'Kernel::System::DateTime',
+                ObjectParams => {
+                    String => $Row[2],
+                },
+            );
+
+            my $Delta = $TicketCreatedDTObj->Delta( DateTimeObject => $CurrentDateTimeObject );
+            my $MaxAge = $Delta->{AbsoluteSeconds};
             $QueueData->{MaxAge} = $MaxAge if $MaxAge > $QueueData->{MaxAge};
 
             # get the oldest queue id
@@ -509,7 +525,7 @@ sub TicketAcceleratorRebuild {
 
     # get all viewable tickets
     my $SQL = "
-        SELECT st.id, st.queue_id, sq.name, sq.group_id, slt.name, tsd.name, st.create_time_unix
+        SELECT st.id, st.queue_id, sq.name, sq.group_id, slt.name, tsd.name, st.create_time
         FROM ticket st
             JOIN queue sq             ON st.queue_id = sq.id
             JOIN ticket_state tsd     ON st.ticket_state_id = tsd.id
@@ -523,13 +539,13 @@ sub TicketAcceleratorRebuild {
     while ( my @Row = $DBObject->FetchrowArray() ) {
 
         my %Data;
-        $Data{TicketID}       = $Row[0];
-        $Data{QueueID}        = $Row[1];
-        $Data{Queue}          = $Row[2];
-        $Data{GroupID}        = $Row[3];
-        $Data{Lock}           = $Row[4];
-        $Data{State}          = $Row[5];
-        $Data{CreateTimeUnix} = $Row[6];
+        $Data{TicketID}   = $Row[0];
+        $Data{QueueID}    = $Row[1];
+        $Data{Queue}      = $Row[2];
+        $Data{GroupID}    = $Row[3];
+        $Data{Lock}       = $Row[4];
+        $Data{State}      = $Row[5];
+        $Data{CreateTime} = $Row[6];
 
         push @RowBuffer, \%Data;
     }
@@ -544,11 +560,11 @@ sub TicketAcceleratorRebuild {
         $DBObject->Do(
             SQL => '
                 INSERT INTO ticket_index
-                    (ticket_id, queue_id, queue, group_id, s_lock, s_state, create_time_unix)
+                    (ticket_id, queue_id, queue, group_id, s_lock, s_state, create_time)
                     VALUES (?, ?, ?, ?, ?, ?, ?)',
             Bind => [
                 \$Data{TicketID}, \$Data{QueueID}, \$Data{Queue}, \$Data{GroupID},
-                \$Data{Lock}, \$Data{State}, \$Data{CreateTimeUnix},
+                \$Data{Lock}, \$Data{State}, \$Data{CreateTime},
             ],
         );
     }
@@ -596,7 +612,7 @@ sub GetIndexTicket {
     # sql query
     return if !$DBObject->Prepare(
         SQL => '
-            SELECT ticket_id, queue_id, queue, group_id, s_lock, s_state, create_time_unix
+            SELECT ticket_id, queue_id, queue, group_id, s_lock, s_state, create_time
             FROM ticket_index
             WHERE ticket_id = ?',
         Bind => [ \$Param{TicketID} ]
@@ -604,13 +620,13 @@ sub GetIndexTicket {
 
     my %Data;
     while ( my @Row = $DBObject->FetchrowArray() ) {
-        $Data{TicketID}       = $Row[0];
-        $Data{QueueID}        = $Row[1];
-        $Data{Queue}          = $Row[2];
-        $Data{GroupID}        = $Row[3];
-        $Data{Lock}           = $Row[4];
-        $Data{State}          = $Row[5];
-        $Data{CreateTimeUnix} = $Row[6];
+        $Data{TicketID}   = $Row[0];
+        $Data{QueueID}    = $Row[1];
+        $Data{Queue}      = $Row[2];
+        $Data{GroupID}    = $Row[3];
+        $Data{Lock}       = $Row[4];
+        $Data{State}      = $Row[5];
+        $Data{CreateTime} = $Row[6];
     }
 
     return %Data;
