@@ -1,9 +1,9 @@
 # --
-# Copyright (C) 2001-2020 OTRS AG, https://otrs.com/
+# Copyright (C) 2001-2017 OTRS AG, http://otrs.com/
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
-# the enclosed file COPYING for license information (GPL). If you
-# did not receive this file, see https://www.gnu.org/licenses/gpl-3.0.txt.
+# the enclosed file COPYING for license information (AGPL). If you
+# did not receive this file, see http://www.gnu.org/licenses/agpl.txt.
 # --
 
 package Kernel::System::User;
@@ -23,7 +23,7 @@ our @ObjectDependencies = (
     'Kernel::System::Log',
     'Kernel::System::Main',
     'Kernel::System::SearchProfile',
-    'Kernel::System::Time',
+    'Kernel::System::DateTime',
     'Kernel::System::Valid',
 );
 
@@ -31,22 +31,16 @@ our @ObjectDependencies = (
 
 Kernel::System::User - user lib
 
-=head1 SYNOPSIS
+=head1 DESCRIPTION
 
 All user functions. E. g. to add and updated user and other functions.
 
 =head1 PUBLIC INTERFACE
 
-=over 4
+=head2 new()
 
-=cut
+Don't use the constructor directly, use the ObjectManager instead:
 
-=item new()
-
-create an object. Do not use it directly, instead use:
-
-    use Kernel::System::ObjectManager;
-    local $Kernel::OM = Kernel::System::ObjectManager->new();
     my $UserObject = $Kernel::OM->Get('Kernel::System::User');
 
 =cut
@@ -79,7 +73,7 @@ sub new {
     return $Self;
 }
 
-=item GetUserData()
+=head2 GetUserData()
 
 get user data (UserLogin, UserFirstname, UserLastname, UserEmail, ...)
 
@@ -249,13 +243,19 @@ sub GetUserData {
     # get preferences
     my %Preferences = $Self->GetPreferences( UserID => $Data{UserID} );
 
-    my $TimeObject = $Kernel::OM->Get('Kernel::System::Time');
+    my $DateTimeObject = $Kernel::OM->Create('Kernel::System::DateTime');
 
     # add last login timestamp
     if ( $Preferences{UserLastLogin} ) {
-        $Preferences{UserLastLoginTimestamp} = $TimeObject->SystemTime2TimeStamp(
-            SystemTime => $Preferences{UserLastLogin},
+
+        my $UserLastLoginTimeObj = $Kernel::OM->Create(
+            'Kernel::System::DateTime',
+            ObjectParams => {
+                Epoch => $Preferences{UserLastLogin}
+                }
         );
+
+        $Preferences{UserLastLoginTimestamp} = $UserLastLoginTimeObj->ToString();
     }
 
     # check compat stuff
@@ -266,18 +266,40 @@ sub GetUserData {
     # out of office check
     if ( !$Param{NoOutOfOffice} ) {
         if ( $Preferences{OutOfOffice} ) {
-            my $Time = $TimeObject->SystemTime();
-            my $Start
-                = "$Preferences{OutOfOfficeStartYear}-$Preferences{OutOfOfficeStartMonth}-$Preferences{OutOfOfficeStartDay} 00:00:00";
-            my $TimeStart = $TimeObject->TimeStamp2SystemTime(
-                String => $Start,
+
+            my $CurrentTimeObject = $Kernel::OM->Create('Kernel::System::DateTime');
+            my $CreateDTObject    = sub {
+                my %Param = @_;
+
+                return $Kernel::OM->Create(
+                    'Kernel::System::DateTime',
+                    ObjectParams => {
+                        String => sprintf(
+                            '%d-%02d-%02d %s',
+                            $Param{Year},
+                            $Param{Month},
+                            $Param{Day},
+                            $Param{Time}
+                        ),
+                    },
+                );
+            };
+
+            my $TimeStartObj = $CreateDTObject->(
+                Year  => $Preferences{OutOfOfficeStartYear},
+                Month => $Preferences{OutOfOfficeStartMonth},
+                Day   => $Preferences{OutOfOfficeStartDay},
+                Time  => '00:00:00',
             );
-            my $End
-                = "$Preferences{OutOfOfficeEndYear}-$Preferences{OutOfOfficeEndMonth}-$Preferences{OutOfOfficeEndDay} 23:59:59";
-            my $TimeEnd = $TimeObject->TimeStamp2SystemTime(
-                String => $End,
+
+            my $TimeEndObj = $CreateDTObject->(
+                Year  => $Preferences{OutOfOfficeEndYear},
+                Month => $Preferences{OutOfOfficeEndMonth},
+                Day   => $Preferences{OutOfOfficeEndDay},
+                Time  => '23:59:59',
             );
-            if ( $TimeStart < $Time && $TimeEnd > $Time ) {
+
+            if ( $TimeStartObj < $CurrentTimeObject && $TimeEndObj > $CurrentTimeObject ) {
                 my $OutOfOfficeMessageTemplate =
                     $ConfigObject->Get('OutOfOfficeMessageTemplate') || '*** out of office until %s (%s d left) ***';
                 my $TillDate = sprintf(
@@ -286,7 +308,7 @@ sub GetUserData {
                     $Preferences{OutOfOfficeEndMonth},
                     $Preferences{OutOfOfficeEndDay}
                 );
-                my $Till = int( ( $TimeEnd - $Time ) / 60 / 60 / 24 );
+                my $Till = int( ( $TimeEndObj->ToEpoch() - $CurrentTimeObject->ToEpoch() ) / 60 / 60 / 24 );
                 $Preferences{OutOfOfficeMessage} = sprintf( $OutOfOfficeMessageTemplate, $TillDate, $Till );
                 $Data{UserFullname} .= ' ' . $Preferences{OutOfOfficeMessage};
             }
@@ -328,7 +350,7 @@ sub GetUserData {
     return %Data;
 }
 
-=item UserAdd()
+=head2 UserAdd()
 
 to add new users
 
@@ -363,7 +385,7 @@ sub UserAdd {
     if ( $Self->UserLoginExistsCheck( UserLogin => $Param{UserLogin} ) ) {
         $Kernel::OM->Get('Kernel::System::Log')->Log(
             Priority => 'error',
-            Message  => "A user with username '$Param{UserLogin}' already exists!"
+            Message  => "A user with the username '$Param{UserLogin}' already exists.",
         );
         return;
     }
@@ -372,7 +394,6 @@ sub UserAdd {
     if (
         $Param{UserEmail}
         && !$Kernel::OM->Get('Kernel::System::CheckItem')->CheckEmail( Address => $Param{UserEmail} )
-        && grep { $_ eq $Param{ValidID} } $Kernel::OM->Get('Kernel::System::Valid')->ValidIDsGet()
         )
     {
         $Kernel::OM->Get('Kernel::System::Log')->Log(
@@ -469,7 +490,7 @@ sub UserAdd {
     return $UserID;
 }
 
-=item UserUpdate()
+=head2 UserUpdate()
 
 to update users
 
@@ -516,7 +537,7 @@ sub UserUpdate {
     {
         $Kernel::OM->Get('Kernel::System::Log')->Log(
             Priority => 'error',
-            Message  => "A user with username '$Param{UserLogin}' already exists!"
+            Message  => "A user with the username '$Param{UserLogin}' already exists.",
         );
         return;
     }
@@ -525,7 +546,6 @@ sub UserUpdate {
     if (
         $Param{UserEmail}
         && !$Kernel::OM->Get('Kernel::System::CheckItem')->CheckEmail( Address => $Param{UserEmail} )
-        && grep { $_ eq $Param{ValidID} } $Kernel::OM->Get('Kernel::System::Valid')->ValidIDsGet()
         )
     {
         $Kernel::OM->Get('Kernel::System::Log')->Log(
@@ -583,7 +603,7 @@ sub UserUpdate {
 
     # TODO Not needed to delete the cache if ValidID or Name was not changed
 
-    my $CacheObject            = $Kernel::OM->Get('Kernel::System::Cache');
+    my $CacheObject = $Kernel::OM->Get('Kernel::System::Cache');
     my $SystemPermissionConfig = $Kernel::OM->Get('Kernel::Config')->Get('System::Permission') || [];
 
     for my $Type ( @{$SystemPermissionConfig}, 'rw' ) {
@@ -601,7 +621,7 @@ sub UserUpdate {
     return 1;
 }
 
-=item UserSearch()
+=head2 UserSearch()
 
 to search users
 
@@ -695,11 +715,10 @@ sub UserSearch {
     }
     elsif ( $Param{UserLogin} ) {
 
-        my $UserLogin = lc $Param{UserLogin};
         $SQL .= " $Self->{Lower}($Self->{UserTableUser}) LIKE ? $LikeEscapeString";
-        $UserLogin =~ s/\*/%/g;
-        $UserLogin = $DBObject->Quote( $UserLogin, 'Like' );
-        push @Bind, \$UserLogin;
+        $Param{UserLogin} =~ s/\*/%/g;
+        $Param{UserLogin} = $DBObject->Quote( $Param{UserLogin}, 'Like' );
+        push @Bind, \$Param{UserLogin};
     }
 
     # add valid option
@@ -723,7 +742,7 @@ sub UserSearch {
     return %Users;
 }
 
-=item SetPassword()
+=head2 SetPassword()
 
 to set users passwords
 
@@ -756,11 +775,12 @@ sub SetPassword {
         return;
     }
 
-    my $Pw        = $Param{PW} || '';
+    my $Pw = $Param{PW} || '';
     my $CryptedPw = '';
 
-    # get crypt type
-    my $CryptType = $Kernel::OM->Get('Kernel::Config')->Get('AuthModule::DB::CryptType') || 'sha2';
+    my $ConfigObject = $Kernel::OM->Get('Kernel::Config');
+
+    my $CryptType = $ConfigObject->Get('AuthModule::DB::CryptType') || 'sha2';
 
     # crypt plain (no crypt at all)
     if ( $CryptType eq 'plain' ) {
@@ -801,10 +821,16 @@ sub SetPassword {
     elsif ( $CryptType eq 'sha1' ) {
 
         my $SHAObject = Digest::SHA->new('sha1');
-
-        # encode output, needed by sha1_hex() only non utf8 signs
         $Kernel::OM->Get('Kernel::System::Encode')->EncodeOutput( \$Pw );
+        $SHAObject->add($Pw);
+        $CryptedPw = $SHAObject->hexdigest();
+    }
 
+    # crypt with sha512
+    elsif ( $CryptType eq 'sha512' ) {
+
+        my $SHAObject = Digest::SHA->new('sha512');
+        $Kernel::OM->Get('Kernel::System::Encode')->EncodeOutput( \$Pw );
         $SHAObject->add($Pw);
         $CryptedPw = $SHAObject->hexdigest();
     }
@@ -821,7 +847,14 @@ sub SetPassword {
             return;
         }
 
-        my $Cost = 9;
+        my $Cost = $ConfigObject->Get('AuthModule::DB::bcryptCost') // 12;
+
+        # Don't allow values smaller than 9 for security.
+        $Cost = 9 if $Cost < 9;
+
+        # Current Crypt::Eksblowfish::Bcrypt limit is 31.
+        $Cost = 31 if $Cost > 31;
+
         my $Salt = $Kernel::OM->Get('Kernel::System::Main')->GenerateRandomString( Length => 16 );
 
         # remove UTF8 flag, required by Crypt::Eksblowfish::Bcrypt
@@ -831,7 +864,7 @@ sub SetPassword {
         my $Octets = Crypt::Eksblowfish::Bcrypt::bcrypt_hash(
             {
                 key_nul => 1,
-                cost    => 9,
+                cost    => $Cost,
                 salt    => $Salt,
             },
             $Pw
@@ -871,7 +904,7 @@ sub SetPassword {
     return 1;
 }
 
-=item UserLookup()
+=head2 UserLookup()
 
 user login or id lookup
 
@@ -995,7 +1028,7 @@ sub UserLookup {
     }
 }
 
-=item UserName()
+=head2 UserName()
 
 get user name
 
@@ -1020,7 +1053,7 @@ sub UserName {
     return $User{UserFullname};
 }
 
-=item UserList()
+=head2 UserList()
 
 return a hash with all users
 
@@ -1045,7 +1078,7 @@ sub UserList {
 
     # get configuration for the full name order
     my $FirstnameLastNameOrder = $ConfigObject->Get('FirstnameLastnameOrder') || 0;
-    my $NoOutOfOffice          = $Param{NoOutOfOffice}                        || 0;
+    my $NoOutOfOffice = $Param{NoOutOfOffice} || 0;
 
     # check cache
     my $CacheKey = join '::', 'UserList', $Type, $Valid, $FirstnameLastNameOrder, $NoOutOfOffice;
@@ -1132,7 +1165,7 @@ sub UserList {
     return %Users;
 }
 
-=item GenerateRandomPassword()
+=head2 GenerateRandomPassword()
 
 generate a random password
 
@@ -1159,7 +1192,7 @@ sub GenerateRandomPassword {
     return $Password;
 }
 
-=item SetPreferences()
+=head2 SetPreferences()
 
 set user preferences
 
@@ -1185,23 +1218,6 @@ sub SetPreferences {
         }
     }
 
-    # Don't allow overwriting of native user data.
-    my %Blacklisted = (
-        UserID        => 1,
-        UserLogin     => 1,
-        UserPw        => 1,
-        UserFirstname => 1,
-        UserLastname  => 1,
-        UserFullname  => 1,
-        UserTitle     => 1,
-        ChangeTime    => 1,
-        CreateTime    => 1,
-        ValidID       => 1,
-    );
-
-    return 0 if $Blacklisted{ $Param{Key} };
-    return 0 if substr( $Param{Key}, 0, 11 ) eq 'UserIsGroup';
-
     # get current setting
     my %User = $Self->GetUserData(
         UserID        => $Param{UserID},
@@ -1238,38 +1254,31 @@ sub _UserCacheClear {
         return;
     }
 
-    # get configuration for the full name order
-    my $FirstnameLastNameOrder = $Kernel::OM->Get('Kernel::Config')->Get('FirstnameLastnameOrder') || 0;
+    my $Login = $Self->UserLookup( UserID => $Param{UserID} );
 
-    # create cachekey
-    my $Login     = $Self->UserLookup( UserID => $Param{UserID} );
-    my @CacheKeys = (
-        'GetUserData::User::' . $Login . '::0::' . $FirstnameLastNameOrder . '::0',
-        'GetUserData::User::' . $Login . '::0::' . $FirstnameLastNameOrder . '::1',
-        'GetUserData::User::' . $Login . '::1::' . $FirstnameLastNameOrder . '::0',
-        'GetUserData::User::' . $Login . '::1::' . $FirstnameLastNameOrder . '::1',
-        'GetUserData::UserID::' . $Param{UserID} . '::0::' . $FirstnameLastNameOrder . '::0',
-        'GetUserData::UserID::' . $Param{UserID} . '::0::' . $FirstnameLastNameOrder . '::1',
-        'GetUserData::UserID::' . $Param{UserID} . '::1::' . $FirstnameLastNameOrder . '::0',
-        'GetUserData::UserID::' . $Param{UserID} . '::1::' . $FirstnameLastNameOrder . '::1',
-        'UserList::Short::0::' . $FirstnameLastNameOrder . '::0',
-        'UserList::Short::0::' . $FirstnameLastNameOrder . '::1',
-        'UserList::Short::1::' . $FirstnameLastNameOrder . '::0',
-        'UserList::Short::1::' . $FirstnameLastNameOrder . '::1',
-        'UserList::Long::0::' . $FirstnameLastNameOrder . '::0',
-        'UserList::Long::0::' . $FirstnameLastNameOrder . '::1',
-        'UserList::Long::1::' . $FirstnameLastNameOrder . '::0',
-        'UserList::Long::1::' . $FirstnameLastNameOrder . '::1',
-        'UserLookup::ID::' . $Login,
-        'UserLookup::Login::' . $Param{UserID},
-    );
+    my @CacheKeys;
+
+    # Delete cache for all possible FirstnameLastNameOrder settings as this might be overridden by users.
+    for my $FirstnameLastNameOrder ( 0 .. 8 ) {
+        for my $ActiveLevel1 ( 0 .. 1 ) {
+            for my $ActiveLevel2 ( 0 .. 1 ) {
+                push @CacheKeys, (
+                    "GetUserData::User::${Login}::${ActiveLevel1}::${FirstnameLastNameOrder}::${ActiveLevel2}",
+                    "GetUserData::UserID::$Param{UserID}::${ActiveLevel1}::${FirstnameLastNameOrder}::${ActiveLevel2}",
+                    "UserList::Short::${ActiveLevel1}::${FirstnameLastNameOrder}::${ActiveLevel2}",
+                    "UserList::Long::${ActiveLevel1}::${FirstnameLastNameOrder}::${ActiveLevel2}",
+                );
+            }
+        }
+        push @CacheKeys, (
+            'UserLookup::ID::' . $Login,
+            'UserLookup::Login::' . $Param{UserID},
+        );
+    }
 
-    # get cache object
     my $CacheObject = $Kernel::OM->Get('Kernel::System::Cache');
 
-    # delete cache
     for my $CacheKey (@CacheKeys) {
-
         $CacheObject->Delete(
             Type => $Self->{CacheType},
             Key  => $CacheKey,
@@ -1279,7 +1288,7 @@ sub _UserCacheClear {
     return 1;
 }
 
-=item GetPreferences()
+=head2 GetPreferences()
 
 get user preferences
 
@@ -1302,7 +1311,7 @@ sub GetPreferences {
     return $PreferencesObject->GetPreferences(%Param);
 }
 
-=item SearchPreferences()
+=head2 SearchPreferences()
 
 search in user preferences
 
@@ -1314,7 +1323,7 @@ search in user preferences
 =cut
 
 sub SearchPreferences {
-    my $Self = shift;
+    my ( $Self, %Param ) = @_;
 
     # get user preferences config
     my $GeneratorModule = $Kernel::OM->Get('Kernel::Config')->Get('User::PreferencesModule')
@@ -1323,10 +1332,10 @@ sub SearchPreferences {
     # get generator preferences module
     my $PreferencesObject = $Kernel::OM->Get($GeneratorModule);
 
-    return $PreferencesObject->SearchPreferences(@_);
+    return $PreferencesObject->SearchPreferences(%Param);
 }
 
-=item TokenGenerate()
+=head2 TokenGenerate()
 
 generate a random token
 
@@ -1361,7 +1370,7 @@ sub TokenGenerate {
     return $Token;
 }
 
-=item TokenCheck()
+=head2 TokenCheck()
 
 check password token
 
@@ -1409,7 +1418,7 @@ sub TokenCheck {
 
 =begin Internal:
 
-=item _UserFullname()
+=head2 _UserFullname()
 
 Builds the user fullname based on firstname, lastname and login. The order
 can be configured.
@@ -1482,6 +1491,9 @@ sub _UserFullname {
             . ') ' . $Param{UserLastname}
             . ' ' . $Param{UserFirstname};
     }
+    elsif ( $FirstnameLastNameOrder eq '9' ) {
+        $UserFullname = $Param{UserLastname} . $Param{UserFirstname};
+    }
     return $UserFullname;
 }
 
@@ -1489,7 +1501,7 @@ sub _UserFullname {
 
 =cut
 
-=item UserLoginExistsCheck()
+=head2 UserLoginExistsCheck()
 
 return 1 if another user with this login (username) already exists
 
@@ -1527,14 +1539,12 @@ sub UserLoginExistsCheck {
 
 1;
 
-=back
-
 =head1 TERMS AND CONDITIONS
 
-This software is part of the OTRS project (L<https://otrs.org/>).
+This software is part of the OTRS project (L<http://otrs.org/>).
 
 This software comes with ABSOLUTELY NO WARRANTY. For details, see
-the enclosed file COPYING for license information (GPL). If you
-did not receive this file, see L<https://www.gnu.org/licenses/gpl-3.0.txt>.
+the enclosed file COPYING for license information (AGPL). If you
+did not receive this file, see L<http://www.gnu.org/licenses/agpl.txt>.
 
 =cut
