@@ -1,9 +1,9 @@
 // --
-// Copyright (C) 2001-2020 OTRS AG, https://otrs.com/
+// Copyright (C) 2001-2017 OTRS AG, http://otrs.com/
 // --
 // This software comes with ABSOLUTELY NO WARRANTY. For details, see
-// the enclosed file COPYING for license information (GPL). If you
-// did not receive this file, see https://www.gnu.org/licenses/gpl-3.0.txt.
+// the enclosed file COPYING for license information (AGPL). If you
+// did not receive this file, see http://www.gnu.org/licenses/agpl.txt.
 // --
 
 /*eslint-disable no-window*/
@@ -23,6 +23,54 @@ Core.Agent.Admin = Core.Agent.Admin || {};
  */
 Core.Agent.Admin.ProcessManagement = (function (TargetNS) {
 
+    /**
+     * @name Init
+     * @memberof Core.Agent.Admin.ProcessManagement
+     * @function
+     * @description
+     *      This function initializes the module functionality.
+     */
+    TargetNS.Init = function () {
+
+        // Initialize Popup response (Redirect and close Popup)
+        InitProcessPopupsResponse();
+
+        // Initialize table filter
+        Core.UI.Table.InitTableFilter($('#Filter'), $('#Processes'), 0);
+
+        // Depending on Subaction initialize specific functions
+        if (Core.Config.Get('Subaction') === 'ActivityNew' ||
+          Core.Config.Get('Subaction') === 'ActivityEdit') {
+            TargetNS.InitActivityEdit();
+        }
+        else if (Core.Config.Get('Subaction') === 'ActivityDialogNew' ||
+          Core.Config.Get('Subaction') === 'ActivityDialogEdit') {
+            TargetNS.InitActivityDialogEdit();
+        }
+        else if (Core.Config.Get('Subaction') === 'TransitionNew' ||
+          Core.Config.Get('Subaction') === 'TransitionEdit') {
+            TargetNS.InitTransitionEdit();
+        }
+        else if (Core.Config.Get('Subaction') === 'TransitionActionNew' ||
+          Core.Config.Get('Subaction') === 'TransitionActionEdit') {
+            TargetNS.InitTransitionActionEdit();
+        }
+        else if (Core.Config.Get('Subaction') === 'ProcessEdit') {
+            TargetNS.InitProcessEdit();
+        }
+        else if (Core.Config.Get('Subaction') === 'ProcessPrint') {
+            $('.ProcessPrint').on('click', function() {
+                window.print();
+                return false;
+            });
+        }
+
+        // Depending on Action initialize specific functions
+        if (Core.Config.Get('Action') === 'AdminProcessManagementPath') {
+           TargetNS.InitPathEdit();
+        }
+    };
+
     /**
      * @private
      * @name InitProcessPopups
@@ -32,7 +80,7 @@ Core.Agent.Admin.ProcessManagement = (function (TargetNS) {
      *      Initializes needed popup handler.
      */
     function InitProcessPopups() {
-        $('a.AsPopup').bind('click', function () {
+        $('a.AsPopup').on('click', function () {
             var Matches,
                 PopupType = 'Process';
 
@@ -49,7 +97,7 @@ Core.Agent.Admin.ProcessManagement = (function (TargetNS) {
             return false;
         });
 
-        $('a.AsPopup_Redirect').bind('click', function () {
+        $('a.AsPopup_Redirect').on('click', function () {
             var $Form = $(this).closest('form');
 
             $('#PopupRedirect').val(1);
@@ -61,10 +109,10 @@ Core.Agent.Admin.ProcessManagement = (function (TargetNS) {
             $('#PopupRedirectStartActivityID').val($(this).data('startactivityid'));
 
             if ($(this).hasClass('Edit_Confirm')) {
-                if (window.confirm(Core.Agent.Admin.ProcessManagement.Localization.EditConfirm)) {
+                if (window.confirm(Core.Language.Translate('As soon as you use this button or link, you will leave this screen and its current state will be saved automatically. Do you want to continue?'))) {
                     // Remove onbeforeunload event only if there is no validation pending on form submit
                     if (!($Form.hasClass("Validate"))) {
-                        $(window).unbind("beforeunload.PMPopup");
+                        $(window).off("beforeunload.PMPopup");
                     }
                     $(this).closest('form').submit();
                 }
@@ -72,19 +120,76 @@ Core.Agent.Admin.ProcessManagement = (function (TargetNS) {
             else {
                 // Remove onbeforeunload event only if there is no validation pending on form submit
                 if (!($Form.hasClass("Validate"))) {
-                    $(window).unbind("beforeunload.PMPopup");
+                    $(window).off("beforeunload.PMPopup");
                 }
                 $(this).closest('form').submit();
             }
             return false;
         });
 
-        $('a.GoBack').bind('click', function () {
+        $('a.GoBack').on('click', function () {
             // Remove onbeforeunload event (which is only needed if you close the popup via the window "X")
-            $(window).unbind("beforeunload.PMPopup");
+            $(window).off("beforeunload.PMPopup");
         });
     }
 
+    /**
+     * @private
+     * @name InitProcessPopupsResponse
+     * @memberof Core.Agent.Admin.ProcessManagement
+     * @function
+     * @description
+     *      Initializes redirect and close popups response.
+     */
+    function InitProcessPopupsResponse() {
+        var Redirect = Core.Config.Get('Redirect'),
+            ClosePopup = Core.Config.Get('ClosePopup'),
+            Data;
+
+        if (typeof Redirect !== 'undefined') {
+
+            Data = {
+                Action: Redirect.Action,
+                Subaction: Redirect.Subaction,
+                ID: Redirect.ID,
+                EntityID: Redirect.EntityID,
+                Field: Redirect.Field,
+                StartActivityID: Redirect.StartActivityID
+            };
+
+            // send results to main window
+            window.opener.Core.Agent.Admin.ProcessManagement.UpdateConfig(Redirect.ConfigJSON);
+
+            // reload popup
+            Core.App.InternalRedirect(Data);
+        }
+        else if (typeof ClosePopup !== 'undefined') {
+
+            window.opener.Core.Agent.Admin.ProcessManagement.UpdateScreensPath(window, function (WindowObject) {
+                //send results to main window
+                WindowObject.opener.Core.Agent.Admin.ProcessManagement.UpdateConfig(ClosePopup.ConfigJSON);
+
+                // update accordion
+                WindowObject.opener.Core.Agent.Admin.ProcessManagement.UpdateAccordion();
+
+                // update sync message
+                WindowObject.opener.Core.Agent.Admin.ProcessManagement.UpdateSyncMessage();
+
+                // redraw canvas
+                WindowObject.opener.Core.Agent.Admin.ProcessManagement.Canvas.Redraw();
+
+                // remove overlay
+                WindowObject.opener.Core.Agent.Admin.ProcessManagement.HideOverlay();
+
+                // remove onbeforeunload event (which is only needed if you close the popup via the window "X")
+                $(WindowObject).off("beforeunload.PMPopup");
+
+                // close popup
+                WindowObject.close();
+            });
+        }
+    }
+
     /**
      * @private
      * @name ShowDeleteProcessConfirmationDialog
@@ -107,14 +212,14 @@ Core.Agent.Admin.ProcessManagement = (function (TargetNS) {
             true,
             [
                {
-                   Label: TargetNS.Localization.CancelMsg,
-                   Class: 'Primary',
+                   Label: Core.Language.Translate('Cancel'),
                    Function: function () {
                        Core.UI.Dialog.CloseDialog($('.Dialog'));
                    }
                },
                {
-                   Label: TargetNS.Localization.DeleteMsg,
+                   Label: Core.Language.Translate('Delete'),
+                   Class: 'Primary',
                    Function: function () {
                        var Data = {
                                Action: 'AdminProcessManagement',
@@ -167,20 +272,20 @@ Core.Agent.Admin.ProcessManagement = (function (TargetNS) {
 
         Core.UI.Dialog.ShowContentDialog(
             $('#Dialogs #' + DialogID),
-            TargetNS.Localization.DeleteEntityTitle,
+            Core.Language.Translate('Delete Entity'),
             '240px',
             'Center',
             true,
             [
                {
-                   Label: TargetNS.Localization.CancelMsg,
-                   Class: 'Primary',
+                   Label: Core.Language.Translate('Cancel'),
                    Function: function () {
                        Core.UI.Dialog.CloseDialog($('.Dialog'));
                    }
                },
                {
-                   Label: TargetNS.Localization.DeleteMsg,
+                   Label: Core.Language.Translate('Delete'),
+                   Class: 'Primary',
                    Function: function () {
                        var Data = {
                                Action: 'AdminProcessManagement',
@@ -222,7 +327,7 @@ Core.Agent.Admin.ProcessManagement = (function (TargetNS) {
      *      Initializes the event handler to delete entities.
      */
     function InitDeleteEntity() {
-        $('a.DeleteEntity').bind('click.DeleteEntity', function () {
+        $('a.DeleteEntity').on('click.DeleteEntity', function () {
             var EntityID = $(this).closest('li').data('entity'),
                 EntityName = $(this).closest('li').clone().children().remove().end().text(),
                 ItemID = $(this).closest('li').data('id'),
@@ -381,7 +486,7 @@ Core.Agent.Admin.ProcessManagement = (function (TargetNS) {
                     TargetNS.Canvas.MakeDraggable();
                 }
                 else {
-                    alert(Core.Agent.Admin.ProcessManagement.Localization.ActivityAlreadyPlaced);
+                    alert(Core.Language.Translate('This Activity is already used in the Process. You cannot add it twice!'));
                 }
             }
             else {
@@ -459,7 +564,7 @@ Core.Agent.Admin.ProcessManagement = (function (TargetNS) {
                                 alert(Response.Message);
                             }
                             else {
-                                alert('Error during AJAX communication');
+                                alert(Core.Language.Translate('Error during AJAX communication'));
                             }
 
                             TargetNS.Canvas.ShowActivityAddActivityDialogError(Activity);
@@ -523,7 +628,7 @@ Core.Agent.Admin.ProcessManagement = (function (TargetNS) {
             // if a dummy activity exists, another transition was placed to the canvas but not yet
             // connected to an end point. One cannot place two unconnected transitions on the canvas.
             if ($('#Dummy').length && DummyActivityConnected(ProcessEntityID)) {
-              alert(Core.Agent.Admin.ProcessManagement.Localization.UnconnectedTransition);
+              alert(Core.Language.Translate('An unconnected transition is already placed on the canvas. Please connect this transition first before placing another transition.'));
               return false;
             }
 
@@ -536,7 +641,7 @@ Core.Agent.Admin.ProcessManagement = (function (TargetNS) {
                 // If this transition is already bind to this activity
                 // you cannot bind it a second time
                 if (Path[Activity] && typeof Path[Activity][EntityID] !== 'undefined') {
-                    alert(Core.Agent.Admin.ProcessManagement.Localization.TransitionAlreadyPlaced);
+                    alert(Core.Language.Translate('This Transition is already used for this Activity. You cannot use it twice!'));
                     return false;
                 }
 
@@ -593,7 +698,7 @@ Core.Agent.Admin.ProcessManagement = (function (TargetNS) {
                     typeof Path[Transition.StartActivity][Transition.TransitionID].TransitionAction !== 'undefined' &&
                     ($.inArray(EntityID, Path[Transition.StartActivity][Transition.TransitionID].TransitionAction) >= 0)
                 ) {
-                    alert(Core.Agent.Admin.ProcessManagement.Localization.TransitionActionAlreadyPlaced);
+                    alert(Core.Language.Translate('This TransitionAction is already used in this Path. You cannot use it twice!'));
                     return false;
                 }
 
@@ -702,8 +807,10 @@ Core.Agent.Admin.ProcessManagement = (function (TargetNS) {
                 Action: 'AdminProcessManagement',
                 Subaction: 'UpdateAccordion'
             },
-            ActiveElementIndex = parseInt($('ul#ProcessElements > li.Active').index(), 10),
-            ActiveElementValue = $('ul#ProcessElements > li:eq(' + ActiveElementIndex + ') .ProcessElementFilter').val();
+            ActiveElement;
+
+        // get active accordion element to open the same element again after updating
+        ActiveElement = $('ul#ProcessElements > li.Active').index();
 
         // Call the ajax function
         Core.AJAX.FunctionCall(Core.Config.Get('CGIHandle'), Data, function (Response) {
@@ -712,8 +819,8 @@ Core.Agent.Admin.ProcessManagement = (function (TargetNS) {
             // Initialize Accordion in the sidebar
             Core.UI.Accordion.Init($('ul#ProcessElements'), 'li.AccordionElement h2 a', 'div.Content');
 
-            // Open accordion element again, which was active before the update.
-            Core.UI.Accordion.OpenElement($('ul#ProcessElements > li:eq(' + ActiveElementIndex + ')'));
+            // open accordion element again, which was active before the update
+            Core.UI.Accordion.OpenElement($('ul#ProcessElements > li:nth-child(' + (parseInt(ActiveElement, 10) + 1) + ')'));
 
             // Initialize filters
             Core.UI.Table.InitTableFilter($('#ActivityFilter'), $('#Activities'));
@@ -721,9 +828,6 @@ Core.Agent.Admin.ProcessManagement = (function (TargetNS) {
             Core.UI.Table.InitTableFilter($('#TransitionFilter'), $('#Transitions'));
             Core.UI.Table.InitTableFilter($('#TransitionActionFilter'), $('#TransitionActions'));
 
-            // Set previous value and filter.
-            $('ul#ProcessElements > li:eq(' + ActiveElementIndex + ') .ProcessElementFilter').trigger('keydown').val(ActiveElementValue);
-
             // Init DnD on Accordion
             TargetNS.InitAccordionDnD();
 
@@ -791,7 +895,7 @@ Core.Agent.Admin.ProcessManagement = (function (TargetNS) {
                     alert(Response.Message);
                 }
                 else {
-                    alert('Error during AJAX communication');
+                    alert(Core.Language.Translate('Error during AJAX communication'));
                 }
                 return false;
             }
@@ -819,18 +923,6 @@ Core.Agent.Admin.ProcessManagement = (function (TargetNS) {
         Core.Agent.Admin.ProcessManagement.HideOverlay();
     };
 
-    /**
-     * @name InitProcessOverview
-     * @memberof Core.Agent.Admin.ProcessManagement
-     * @function
-     * @description
-     *      Initialize process overview screen.
-     */
-    TargetNS.InitProcessOverview = function() {
-        InitProcessPopups();
-        Core.UI.Table.InitTableFilter($('#Filter'), $('#Processes'), 0);
-    };
-
     /**
      * @name InitProcessEdit
      * @memberof Core.Agent.Admin.ProcessManagement
@@ -839,16 +931,18 @@ Core.Agent.Admin.ProcessManagement = (function (TargetNS) {
      *      Initialize process edit screen.
      */
     TargetNS.InitProcessEdit = function () {
+      var ConfigProcess = Core.Config.Get('ConfigProcess');
+
         // Get Process Data
         TargetNS.ProcessData = {
-            Process: Core.Config.Get('Config.Process'),
-            Activity: Core.Config.Get('Config.Activity'),
-            ActivityDialog: Core.Config.Get('Config.ActivityDialog'),
-            Transition: Core.Config.Get('Config.Transition'),
-            TransitionAction: Core.Config.Get('Config.TransitionAction')
+            Process: ConfigProcess.Process,
+            Activity: ConfigProcess.Activity,
+            ActivityDialog: ConfigProcess.ActivityDialog,
+            Transition: ConfigProcess.Transition,
+            TransitionAction: ConfigProcess.TransitionAction
         };
 
-        TargetNS.ProcessLayout = Core.Config.Get('Config.ProcessLayout');
+        TargetNS.ProcessLayout = ConfigProcess.ProcessLayout;
 
         // Initialize Accordion in the sidebar
         Core.UI.Accordion.Init($('ul#ProcessElements'), 'li.AccordionElement h2 a', 'div.Content');
@@ -869,19 +963,14 @@ Core.Agent.Admin.ProcessManagement = (function (TargetNS) {
         InitDeleteEntity();
 
         // Initialize DeleteProcess
-        $('#ProcessDelete').bind('click.ProcessDelete', function (Event) {
+        $('#ProcessDelete').on('click.ProcessDelete', function (Event) {
             ShowDeleteProcessConfirmationDialog($(Event.target).closest('a'));
             Event.stopPropagation();
             return false;
         });
 
-        $('#SubmitAndContinue').bind('click', function() {
-            $('#ContinueAfterSave').val(1);
-            $('#Submit').click();
-        });
-
         // Init submit function
-        $('#Submit').bind('click', function () {
+        $('#Submit').on('click', function () {
             var ProcessEntityID = $('#ProcessEntityID').val(),
                 StartActivity;
 
@@ -912,7 +1001,7 @@ Core.Agent.Admin.ProcessManagement = (function (TargetNS) {
         });
 
         // Init Canvas Resizing Functions
-        $('#ExtendCanvasHeight').bind('click', function () {
+        $('#ExtendCanvasHeight').on('click', function () {
             TargetNS.Canvas.Extend({
                 Width: 0,
                 Height: 150
@@ -920,7 +1009,7 @@ Core.Agent.Admin.ProcessManagement = (function (TargetNS) {
             return false;
         });
 
-        $('#ExtendCanvasWidth').bind('click', function () {
+        $('#ExtendCanvasWidth').on('click', function () {
             TargetNS.Canvas.Extend({
                 Width: 150,
                 Height: 0
@@ -928,13 +1017,13 @@ Core.Agent.Admin.ProcessManagement = (function (TargetNS) {
             return false;
         });
 
-        $('#ShowEntityIDs').bind('click', function() {
+        $('#ShowEntityIDs').on('click', function() {
             if ($(this).hasClass('Visible')) {
-                $(this).removeClass('Visible').text(Core.Agent.Admin.ProcessManagement.Localization.ShowEntityIDs);
+                $(this).removeClass('Visible').text(Core.Language.Translate('Show EntityIDs'));
                 $('em.EntityID').remove();
             }
             else {
-                $(this).addClass('Visible').text(Core.Agent.Admin.ProcessManagement.Localization.HideEntityIDs);
+                $(this).addClass('Visible').text(Core.Language.Translate('Hide EntityIDs'));
                 TargetNS.Canvas.ShowEntityIDs();
             }
             return false;
@@ -965,7 +1054,7 @@ Core.Agent.Admin.ProcessManagement = (function (TargetNS) {
         // Initialize list filter
         Core.UI.Table.InitTableFilter($('#FilterAvailableActivityDialogs'), $('#AvailableActivityDialogs'));
 
-        $('#Submit').bind('click', function() {
+        $('#Submit').on('click', function() {
             $('#ActivityForm').submit();
         });
 
@@ -975,7 +1064,7 @@ Core.Agent.Admin.ProcessManagement = (function (TargetNS) {
             $('input[name=ActivityDialogs]').val(Core.JSON.Stringify(Core.UI.AllocationList.GetResult('#AssignedActivityDialogs', 'id')));
 
             // not needed for normal submit
-            $(window).unbind("beforeunload.PMPopup");
+            $(window).off("beforeunload.PMPopup");
 
             Form.submit();
         });
@@ -984,12 +1073,12 @@ Core.Agent.Admin.ProcessManagement = (function (TargetNS) {
         InitProcessPopups();
 
         // Init handling of closing popup with the OS functionality ("X")
-        $(window).unbind("beforeunload.PMPopup").bind("beforeunload.PMPopup", function () {
+        $(window).off("beforeunload.PMPopup").on("beforeunload.PMPopup", function () {
             window.opener.Core.Agent.Admin.ProcessManagement.HandlePopupClose();
         });
 
-        $('.ClosePopup').bind("click", function () {
-            $(window).unbind("beforeunload.PMPopup");
+        $('.ClosePopup').on("click", function () {
+            $(window).off("beforeunload.PMPopup");
         });
     };
 
@@ -1039,7 +1128,7 @@ Core.Agent.Admin.ProcessManagement = (function (TargetNS) {
         // Initialize list filter
         Core.UI.Table.InitTableFilter($('#FilterAvailableFields'), $('#AvailableFields'));
 
-        $('#Submit').bind('click', function() {
+        $('#Submit').on('click', function() {
             $('#ActivityDialogForm').submit();
             return false;
         });
@@ -1069,13 +1158,13 @@ Core.Agent.Admin.ProcessManagement = (function (TargetNS) {
             $('input[name=FieldDetails]').val(Core.JSON.Stringify(FieldDetails));
 
             // not needed for normal submit
-            $(window).unbind("beforeunload.PMPopup");
+            $(window).off("beforeunload.PMPopup");
 
             Form.submit();
         });
 
         // Init Fields modal overlay
-        $('.FieldDetailsOverlay').unbind('click').bind('click', function () {
+        $('.FieldDetailsOverlay').off('click').on('click', function () {
             var FieldConfig = $(this).closest('li').data('config'),
                 $Element = $(this),
                 Fieldname = $.trim($(this).closest('li').data('id')),
@@ -1088,13 +1177,13 @@ Core.Agent.Admin.ProcessManagement = (function (TargetNS) {
             // Open dialog
             Core.UI.Dialog.ShowContentDialog(
                 $('#Dialogs #FieldDetails'),
-                TargetNS.Localization.DialogTitle + ': ' + FieldNameTranslated,
+                Core.Language.Translate('Edit Field Details') + ': ' + FieldNameTranslated,
                 '200px',
                 'Center',
                 true,
                 [
                      {
-                         Label: TargetNS.Localization.SaveMsg,
+                         Label: Core.Language.Translate('Save'),
                          Class: 'Primary',
                          Function: function () {
                              var FieldConfigElement = {};
@@ -1108,11 +1197,16 @@ Core.Agent.Admin.ProcessManagement = (function (TargetNS) {
                                  if (typeof FieldConfigElement.Config === 'undefined'){
                                      FieldConfigElement.Config = {};
                                  }
-                                 FieldConfigElement.Config.ArticleType = $('#ArticleType').val();
+                                 FieldConfigElement.Config.CommunicationChannel = $('#CommunicationChannel').val();
 
-                                 // show error if internal article type is set for an interface different than AgentInterface
-                                 if ($('#Interface').val() !== 'AgentInterface' && $('#ArticleType').val().match(/-int/i)){
-                                     window.alert(Core.Agent.Admin.ProcessManagement.Localization.WrongArticleTypeMsg);
+                                 FieldConfigElement.Config.IsVisibleForCustomer = '0';
+                                 if ($('#IsVisibleForCustomer').prop('checked')) {
+                                    FieldConfigElement.Config.IsVisibleForCustomer = '1';
+                                 }
+
+                                 // show error if not customer visible article is set for an interface different than AgentInterface
+                                 if ($('#Interface').val() !== 'AgentInterface' && !$('#IsVisibleForCustomer').prop('checked')){
+                                     window.alert(Core.Language.Translate('Customer interface does not support articles not visible for customers.'));
                                      return false;
                                  }
 
@@ -1126,7 +1220,7 @@ Core.Agent.Admin.ProcessManagement = (function (TargetNS) {
                          }
                      },
                      {
-                         Label: TargetNS.Localization.CancelMsg,
+                         Label: Core.Language.Translate('Cancel'),
                          Function: function () {
                              Core.UI.Dialog.CloseDialog($('.Dialog'));
                          }
@@ -1167,8 +1261,11 @@ Core.Agent.Admin.ProcessManagement = (function (TargetNS) {
                     if (typeof FieldConfig.Config === 'undefined'){
                         FieldConfig.Config = {};
                     }
-                    if (FieldConfig.Config.ArticleType) {
-                        $('#ArticleType').val(FieldConfig.Config.ArticleType);
+                    if (FieldConfig.Config.CommunicationChannel) {
+                        $('#CommunicationChannel').val(FieldConfig.Config.CommunicationChannel);
+                    }
+                    if (FieldConfig.Config.IsVisibleForCustomer === '1') {
+                        $('#IsVisibleForCustomer').prop("checked", true);
                     }
                     if (FieldConfig.Config.TimeUnits) {
                         $('#TimeUnits').val(FieldConfig.Config.TimeUnits);
@@ -1185,12 +1282,15 @@ Core.Agent.Admin.ProcessManagement = (function (TargetNS) {
                 $('#DefaultValue').prop('readonly', true).prop('disabled', true);
             }
 
-            // only article should show ArticleType select.
+            // only article should show Communication channel select.
             if (Fieldname === 'Article') {
 
-                $('#ArticleTypeContainer').removeClass('Hidden');
-                $('#ArticleTypeContainer').prev('label').css('display', 'block');
-                $('#ArticleTypeContainer .Modernize').trigger('redraw.InputField');
+                $('#CommunicationChannelContainer').removeClass('Hidden');
+                $('#CommunicationChannelContainer').prev('label').css('display', 'block');
+                $('#CommunicationChannelContainer .Modernize').trigger('redraw.InputField');
+
+                $('#IsVisibleForCustomerContainer').removeClass('Hidden');
+                $('#IsVisibleForCustomerContainer').prev('label').css('display', 'block');
 
                 $('#TimeUnitsContainer').removeClass('Hidden');
                 $('#TimeUnitsContainer').prev('label').css('display', 'block');
@@ -1198,8 +1298,11 @@ Core.Agent.Admin.ProcessManagement = (function (TargetNS) {
             }
             else {
 
-                $('#ArticleTypeContainer').addClass('Hidden');
-                $('#ArticleTypeContainer').prev('label').css('display', 'none');
+                $('#CommunicationChannelContainer').addClass('Hidden');
+                $('#CommunicationChannelContainer').prev('label').css('display', 'none');
+
+                $('#IsVisibleForCustomerContainer').addClass('Hidden');
+                $('#IsVisibleForCustomerContainer').prev('label').css('display', 'none');
 
                 $('#TimeUnitsContainer').addClass('Hidden');
                 $('#TimeUnitsContainer').prev('label').css('display', 'none');
@@ -1211,12 +1314,12 @@ Core.Agent.Admin.ProcessManagement = (function (TargetNS) {
         InitProcessPopups();
 
         // Init handling of closing popup with the OS functionality ("X")
-        $(window).unbind("beforeunload.PMPopup").bind("beforeunload.PMPopup", function () {
+        $(window).off("beforeunload.PMPopup").on("beforeunload.PMPopup", function () {
             window.opener.Core.Agent.Admin.ProcessManagement.HandlePopupClose();
         });
 
-        $('.ClosePopup').bind("click", function () {
-            $(window).unbind("beforeunload.PMPopup");
+        $('.ClosePopup').on("click", function () {
+            $(window).off("beforeunload.PMPopup");
         });
     };
 
@@ -1230,7 +1333,7 @@ Core.Agent.Admin.ProcessManagement = (function (TargetNS) {
     TargetNS.InitTransitionEdit = function () {
 
         // Init addition of new conditions
-        $('#ConditionAdd').bind('click', function() {
+        $('#ConditionAdd').on('click', function() {
             // get current parent index
             var CurrentParentIndex = parseInt($(this).prev('.WidgetSimple').first().attr('id').replace(/Condition\[/g, '').replace(/\]/g, ''), 10),
                 // in case we add a whole new condition, the fieldindex must be 1
@@ -1250,7 +1353,7 @@ Core.Agent.Admin.ProcessManagement = (function (TargetNS) {
                 $(this).closest('.WidgetSimple').remove();
             }
             else {
-                alert("Sorry, the only existing condition can't be removed.");
+                alert(Core.Language.Translate("Sorry, the only existing condition can't be removed."));
             }
 
             return false;
@@ -1278,13 +1381,13 @@ Core.Agent.Admin.ProcessManagement = (function (TargetNS) {
                 $(this).parent().closest('fieldset').remove();
             }
             else {
-                alert("Sorry, the only existing field can't be removed.");
+                alert(Core.Language.Translate("Sorry, the only existing field can't be removed."));
             }
 
             return false;
         });
 
-        $('#Submit').bind('click', function() {
+        $('#Submit').on('click', function() {
             $('#TransitionForm').submit();
             return false;
         });
@@ -1294,18 +1397,18 @@ Core.Agent.Admin.ProcessManagement = (function (TargetNS) {
             $('input[name=ConditionConfig]').val(Core.JSON.Stringify(ConditionConfig));
 
             // not needed for normal submit
-            $(window).unbind("beforeunload.PMPopup");
+            $(window).off("beforeunload.PMPopup");
 
             Form.submit();
         });
 
         // Init handling of closing popup with the OS functionality ("X")
-        $(window).unbind("beforeunload.PMPopup").bind("beforeunload.PMPopup", function () {
+        $(window).off("beforeunload.PMPopup").on("beforeunload.PMPopup", function () {
             window.opener.Core.Agent.Admin.ProcessManagement.HandlePopupClose();
         });
 
-        $('.ClosePopup').bind("click", function () {
-            $(window).unbind("beforeunload.PMPopup");
+        $('.ClosePopup').on("click", function () {
+            $(window).off("beforeunload.PMPopup");
         });
 
         Core.UI.InputFields.Init();
@@ -1321,7 +1424,7 @@ Core.Agent.Admin.ProcessManagement = (function (TargetNS) {
      */
     TargetNS.InitTransitionActionEdit = function () {
         // Init addition of new config parameters
-        $('#ConfigAdd').bind('click', function() {
+        $('#ConfigAdd').on('click', function() {
             // get the index for the newly to be added element
             // therefore, we search the preceding fieldset and the first
             // label in it to get its "for"-attribute which contains the index
@@ -1349,19 +1452,19 @@ Core.Agent.Admin.ProcessManagement = (function (TargetNS) {
                 $(this).closest('fieldset').remove();
             }
             else {
-                alert("Sorry, the only existing parameter can't be removed.");
+                alert(Core.Language.Translate("Sorry, the only existing parameter can't be removed."));
             }
             return false;
         });
 
-        $('#Submit').bind('click', function() {
+        $('#Submit').on('click', function() {
             $('#TransitionForm').submit();
             return false;
         });
 
         Core.Form.Validate.SetSubmitFunction($('#TransitionForm'), function (Form) {
             // not needed for normal submit
-            $(window).unbind("beforeunload.PMPopup");
+            $(window).off("beforeunload.PMPopup");
 
             Form.submit();
         });
@@ -1369,12 +1472,12 @@ Core.Agent.Admin.ProcessManagement = (function (TargetNS) {
         InitProcessPopups();
 
         // Init handling of closing popup with the OS functionality ("X")
-        $(window).unbind("beforeunload.PMPopup").bind("beforeunload.PMPopup", function () {
+        $(window).off("beforeunload.PMPopup").on("beforeunload.PMPopup", function () {
             window.opener.Core.Agent.Admin.ProcessManagement.HandlePopupClose();
         });
 
-        $('.ClosePopup').bind("click", function () {
-            $(window).unbind("beforeunload.PMPopup");
+        $('.ClosePopup').on("click", function () {
+            $(window).off("beforeunload.PMPopup");
         });
     };
 
@@ -1386,9 +1489,9 @@ Core.Agent.Admin.ProcessManagement = (function (TargetNS) {
      *      Initialize path edit screen.
      */
     TargetNS.InitPathEdit = function () {
-        var CurrentProcessEntityID = Core.Config.Get('Config.ProcessEntityID'),
-            CurrentTransitionEntityID = Core.Config.Get('Config.TransitionEntityID'),
-            StartActivityID = Core.Config.Get('Config.StartActivityID'),
+        var CurrentProcessEntityID = Core.Config.Get('ProcessEntityID'),
+            CurrentTransitionEntityID = Core.Config.Get('TransitionEntityID'),
+            StartActivityID = Core.Config.Get('StartActivityID'),
             ActivityInfo = window.opener.Core.Agent.Admin.ProcessManagement.ProcessData.Activity,
             PathInfo = window.opener.Core.Agent.Admin.ProcessManagement.ProcessData.Process[CurrentProcessEntityID].Path,
             StartActivityEntityID = '', EndActivityEntityID = '',
@@ -1429,11 +1532,11 @@ Core.Agent.Admin.ProcessManagement = (function (TargetNS) {
         }
 
         // if transition select is updated, also update transition edit link
-        $('#Transition').bind('change', function () {
+        $('#Transition').on('change', function () {
            $('#EditPath a').data('entity', $(this).val());
         });
 
-        $('#Submit').bind('click', function() {
+        $('#Submit').on('click', function() {
             $('#PathForm').submit();
             return false;
         });
@@ -1460,7 +1563,7 @@ Core.Agent.Admin.ProcessManagement = (function (TargetNS) {
             $('#TransitionInfo').val(Core.JSON.Stringify(TransitionInfo));
 
             // not needed for normal submit
-            $(window).unbind("beforeunload.PMPopup");
+            $(window).off("beforeunload.PMPopup");
 
             Form.submit();
         });
@@ -1469,12 +1572,12 @@ Core.Agent.Admin.ProcessManagement = (function (TargetNS) {
         InitProcessPopups();
 
         // Init handling of closing popup with the OS functionality ("X")
-        $(window).unbind("beforeunload.PMPopup").bind("beforeunload.PMPopup", function () {
+        $(window).off("beforeunload.PMPopup").on("beforeunload.PMPopup", function () {
             window.opener.Core.Agent.Admin.ProcessManagement.HandlePopupClose();
         });
 
-        $('.ClosePopup').bind("click", function () {
-            $(window).unbind("beforeunload.PMPopup");
+        $('.ClosePopup').on("click", function () {
+            $(window).off("beforeunload.PMPopup");
         });
     };
 
@@ -1609,6 +1712,8 @@ Core.Agent.Admin.ProcessManagement = (function (TargetNS) {
         }
     };
 
+    Core.Init.RegisterNamespace(TargetNS, 'APP_MODULE');
+
     return TargetNS;
 }(Core.Agent.Admin.ProcessManagement || {}));
 
